@@ -16,7 +16,7 @@
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "pch.h"
 #include "db.h"
 #include "dbhelpers.h"
 #include "query.h"
@@ -24,10 +24,11 @@
 #include "queryoptimizer.h"
 #include "btree.h"
 #include "pdfile.h"
+#include "oplog.h"
 
 namespace mongo {
 
-    CursorIterator::CursorIterator( auto_ptr<Cursor> c , BSONObj filter )
+    CursorIterator::CursorIterator( shared_ptr<Cursor> c , BSONObj filter )
         : _cursor( c ){
             if ( ! filter.isEmpty() )
                 _matcher.reset( new CoveredIndexMatcher( filter , BSONObj() ) );
@@ -93,51 +94,65 @@ namespace mongo {
     class FindOne : public QueryOp {
     public:
         FindOne( bool requireIndex ) : requireIndex_( requireIndex ) {}
-        virtual void init() {
+        virtual void _init() {
             if ( requireIndex_ && strcmp( qp().indexKey().firstElement().fieldName(), "$natural" ) == 0 )
                 throw MsgAssertionException( 9011 , "Not an index cursor" );
             c_ = qp().newCursor();
-            if ( !c_->ok() )
+            if ( !c_->ok() ) {
                 setComplete();
-            else
-                matcher_.reset( new CoveredIndexMatcher( qp().query(), qp().indexKey() ) );
+            }
         }
         virtual void next() {
             if ( !c_->ok() ) {
                 setComplete();
                 return;
             }
-            if ( matcher_->matches( c_->currKey(), c_->currLoc() ) ) {
+            if ( matcher()->matches( c_->currKey(), c_->currLoc() ) ) {
                 one_ = c_->current();
-                setComplete();
+                loc_ = c_->currLoc();
+                setStop();
             } else {
                 c_->advance();
             }
         }
         virtual bool mayRecordPlan() const { return false; }
-        virtual QueryOp *clone() const { return new FindOne( requireIndex_ ); }
+        virtual QueryOp *_createChild() const { return new FindOne( requireIndex_ ); }
         BSONObj one() const { return one_; }
+        DiskLoc loc() const { return loc_; }
     private:
         bool requireIndex_;
-        auto_ptr< Cursor > c_;
-        auto_ptr< CoveredIndexMatcher > matcher_;
+        shared_ptr<Cursor> c_;
         BSONObj one_;
+        DiskLoc loc_;
     };
     
     /* fetch a single object from collection ns that matches query 
        set your db SavedContext first
     */
-    bool Helpers::findOne(const char *ns, BSONObj query, BSONObj& result, bool requireIndex) { 
-        QueryPlanSet s( ns, query, BSONObj(), 0, !requireIndex );
+    bool Helpers::findOne(const char *ns, const BSONObj &query, BSONObj& result, bool requireIndex) { 
+        MultiPlanScanner s( ns, query, BSONObj(), 0, !requireIndex );
         FindOne original( requireIndex );
         shared_ptr< FindOne > res = s.runOp( original );
-        massert( 10302 ,  res->exceptionMessage(), res->complete() );
+        if ( ! res->complete() )
+            throw MsgAssertionException( res->exception() );
         if ( res->one().isEmpty() )
             return false;
         result = res->one();
         return true;
     }
 
+    /* fetch a single object from collection ns that matches query 
+       set your db SavedContext first
+    */
+    DiskLoc Helpers::findOne(const char *ns, const BSONObj &query, bool requireIndex) { 
+        MultiPlanScanner s( ns, query, BSONObj(), 0, !requireIndex );
+        FindOne original( requireIndex );
+        shared_ptr< FindOne > res = s.runOp( original );
+        if ( ! res->complete() )
+            throw MsgAssertionException( res->exception() );
+        return res->loc();
+    }
+
     auto_ptr<CursorIterator> Helpers::find( const char *ns , BSONObj query , bool requireIndex ){
         uassert( 10047 ,  "requireIndex not supported in Helpers::find yet" , ! requireIndex );
         auto_ptr<CursorIterator> i;
@@ -145,9 +160,9 @@ namespace mongo {
         return i;
     }
     
-    
     bool Helpers::findById(Client& c, const char *ns, BSONObj query, BSONObj& result ,
                            bool * nsFound , bool * indexFound ){
+        dbMutex.assertAtLeastReadLocked();
         Database *database = c.database();
         assert( database );
         NamespaceDetails *d = database->namespaceIndex.details(ns);
@@ -173,6 +188,20 @@ namespace mongo {
         return true;
     }
 
+     DiskLoc Helpers::findById(NamespaceDetails *d, BSONObj idquery) {
+         int idxNo = d->findIdIndex();
+         uassert(13430, "no _id index", idxNo>=0);
+         IndexDetails& i = d->idx( idxNo );        
+         BSONObj key = i.getKeyFromQuery( idquery );
+         return i.head.btree()->findSingle( i , i.head , key );
+    }
+
+    bool Helpers::isEmpty(const char *ns) {
+        Client::Context context(ns);
+        shared_ptr<Cursor> c = DataFileMgr::findAll(ns);
+        return !c->ok();
+    }
+
     /* Get the first object from a collection.  Generally only useful if the collection
        only ever has a single object -- which is a "singleton collection.
 
@@ -181,7 +210,7 @@ namespace mongo {
     bool Helpers::getSingleton(const char *ns, BSONObj& result) {
         Client::Context context(ns);
 
-        auto_ptr<Cursor> c = DataFileMgr::findAll(ns);
+        shared_ptr<Cursor> c = DataFileMgr::findAll(ns);
         if ( !c->ok() )
             return false;
 
@@ -189,10 +218,92 @@ namespace mongo {
         return true;
     }
 
+    bool Helpers::getLast(const char *ns, BSONObj& result) {
+        Client::Context ctx(ns);
+        shared_ptr<Cursor> c = findTableScan(ns, reverseNaturalObj);
+        if( !c->ok() ) 
+            return false;
+        result = c->current();
+        return true;
+    }
+
+    void Helpers::upsert( const string& ns , const BSONObj& o ){
+        BSONElement e = o["_id"];
+        assert( e.type() );
+        BSONObj id = e.wrap();
+        
+        OpDebug debug;
+        Client::Context context(ns);
+        updateObjects(ns.c_str(), o, /*pattern=*/id, /*upsert=*/true, /*multi=*/false , /*logtheop=*/true , debug );
+    }
+
     void Helpers::putSingleton(const char *ns, BSONObj obj) {
         OpDebug debug;
         Client::Context context(ns);
-        updateObjects(ns, obj, /*pattern=*/BSONObj(), /*upsert=*/true, /*multi=*/false , true , debug );
+        updateObjects(ns, obj, /*pattern=*/BSONObj(), /*upsert=*/true, /*multi=*/false , /*logtheop=*/true , debug );
+    }
+
+    void Helpers::putSingletonGod(const char *ns, BSONObj obj, bool logTheOp) {
+        OpDebug debug;
+        Client::Context context(ns);
+        _updateObjects(/*god=*/true, ns, obj, /*pattern=*/BSONObj(), /*upsert=*/true, /*multi=*/false , logTheOp , debug );
+    }
+
+    BSONObj Helpers::toKeyFormat( const BSONObj& o , BSONObj& key ){
+        BSONObjBuilder me;
+        BSONObjBuilder k;
+
+        BSONObjIterator i( o );
+        while ( i.more() ){
+            BSONElement e = i.next();
+            k.append( e.fieldName() , 1 );
+            me.appendAs( e , "" );
+        }
+        key = k.obj();
+        return me.obj();
+    }
+    
+    long long Helpers::removeRange( const string& ns , const BSONObj& min , const BSONObj& max , bool yield , bool maxInclusive , RemoveCallback * callback ){
+        BSONObj keya , keyb;
+        BSONObj minClean = toKeyFormat( min , keya );
+        BSONObj maxClean = toKeyFormat( max , keyb );
+        assert( keya == keyb );
+
+        Client::Context ctx(ns);
+        NamespaceDetails* nsd = nsdetails( ns.c_str() );
+        if ( ! nsd )
+            return 0;
+
+        int ii = nsd->findIndexByKeyPattern( keya );
+        assert( ii >= 0 );
+        
+        long long num = 0;
+        
+        IndexDetails& i = nsd->idx( ii );
+
+        shared_ptr<Cursor> c( new BtreeCursor( nsd , ii , i , minClean , maxClean , maxInclusive, 1 ) );
+        auto_ptr<ClientCursor> cc( new ClientCursor( QueryOption_NoCursorTimeout , c , ns ) );
+        cc->setDoingDeletes( true );
+        
+        while ( c->ok() ){
+            DiskLoc rloc = c->currLoc();
+            BSONObj key = c->currKey();
+
+            if ( callback )
+                callback->goingToDelete( c->current() );
+            
+            c->advance();
+            c->noteLocation();
+            
+            logOp( "d" , ns.c_str() , rloc.obj()["_id"].wrap() );
+            theDataFileMgr.deleteRecord(ns.c_str() , rloc.rec(), rloc);
+            num++;
+
+            c->checkLocation();
+
+        }
+
+        return num;
     }
 
     void Helpers::emptyCollection(const char *ns) {
@@ -241,7 +352,7 @@ namespace mongo {
         if ( val ) {
             try {
                 BSONObj k = obj;
-                theDataFileMgr.insert( name_.c_str(), k, false );
+                theDataFileMgr.insertWithObjMod( name_.c_str(), k, false );
             } catch ( DBException& ) {
                 // dup key - already in set
             }
@@ -249,5 +360,48 @@ namespace mongo {
             deleteObjects( name_.c_str(), obj, true, false, false );
         }                        
     }
+
+    RemoveSaver::RemoveSaver( const string& a , const string& b , const string& why) : _out(0){
+        static int NUM = 0;
+        
+        _root = dbpath;
+        if ( a.size() )
+            _root /= a;
+        if ( b.size() )
+            _root /= b;
+        assert( a.size() || b.size() );
+        
+        _file = _root;
+        
+        stringstream ss;
+        ss << why << "." << terseCurrentTime(false) << "." << NUM++ << ".bson";
+        _file /= ss.str();
+
+    }
+    
+    RemoveSaver::~RemoveSaver(){
+        if ( _out ){
+            _out->close();
+            delete _out;
+            _out = 0;
+        }
+    }
+    
+    void RemoveSaver::goingToDelete( const BSONObj& o ){
+        if ( ! _out ){
+            create_directories( _root );
+            _out = new ofstream();
+            _out->open( _file.string().c_str() , ios_base::out | ios_base::binary );
+            if ( ! _out->good() ){
+                log( LL_WARNING ) << "couldn't create file: " << _file.string() << " for remove saving" << endl;
+                delete _out;
+                _out = 0;
+                return;
+            }
+            
+        }
+        _out->write( o.objdata() , o.objsize() );
+    }
+    
         
 } // namespace mongo
