@@ -15,7 +15,7 @@
  *    limitations under the License.
  */
 
-#include "stdafx.h"
+#include "pch.h"
 #include "nonce.h"
 
 extern int do_md5_test(void);
@@ -32,7 +32,7 @@ namespace mongo {
 		if( _initialized ) return;
 		_initialized = true;
 
-#if defined(__linux__)
+#if defined(__linux__) || defined(__sunos__)
         _devrandom = new ifstream("/dev/urandom", ios::binary|ios::in);
         massert( 10353 ,  "can't open dev/urandom", _devrandom->is_open() );
 #elif defined(_WIN32)
@@ -49,7 +49,7 @@ namespace mongo {
     }
     
     nonce Security::getNonce(){
-        static mongo::mutex m;
+        static mongo::mutex m("getNonce");
         scoped_lock lk(m);
 
 		/* question/todo: /dev/random works on OS X.  is it better 
@@ -57,7 +57,7 @@ namespace mongo {
 		*/
 
         nonce n;
-#if defined(__linux__)
+#if defined(__linux__) || defined(__sunos__)
         _devrandom->read((char*)&n, sizeof(n));
         massert( 10355 , "devrandom failed", !_devrandom->fail());
 #elif defined(_WIN32)
@@ -67,6 +67,7 @@ namespace mongo {
 #endif
         return n;
     }
+    unsigned getRandomNumber() { return (unsigned) security.getNonce(); }
     
 	bool Security::_initialized;
     Security security;
