@@ -1,14 +1,18 @@
 // key_many.js
 
 // values have to be sorted
+// you must have exactly 6 values in each array
 types =  [
     { name : "string" , values : [ "allan" , "bob" , "eliot" , "joe" , "mark" , "sara" ] , keyfield: "k" } ,
     { name : "double" , values : [ 1.2 , 3.5 , 4.5 , 4.6 , 6.7 , 9.9 ] , keyfield : "a" } ,
     { name : "date" , values : [ new Date( 1000000 ) , new Date( 2000000 ) , new Date( 3000000 ) , new Date( 4000000 ) , new Date( 5000000 ) , new Date( 6000000 )  ] , keyfield : "a" } ,
     { name : "string_id" , values : [ "allan" , "bob" , "eliot" , "joe" , "mark" , "sara" ] , keyfield : "_id" },
-    { name : "embedded" , values : [ "allan" , "bob" , "eliot" , "joe" , "mark" , "sara" ] , keyfield : "a.b" } ,
+    { name : "embedded 1" , values : [ "allan" , "bob" , "eliot" , "joe" , "mark" , "sara" ] , keyfield : "a.b" } ,
     { name : "embedded 2" , values : [ "allan" , "bob" , "eliot" , "joe" , "mark" , "sara" ] , keyfield : "a.b.c" } ,
     { name : "object" , values : [ {a:1, b:1.2}, {a:1, b:3.5}, {a:1, b:4.5}, {a:2, b:1.2}, {a:2, b:3.5}, {a:2, b:4.5} ] , keyfield : "o" } ,
+    { name : "compound" , values : [ {a:1, b:1.2}, {a:1, b:3.5}, {a:1, b:4.5}, {a:2, b:1.2}, {a:2, b:3.5}, {a:2, b:4.5} ] , keyfield : "o" , compound : true } ,
+    { name : "oid_id" , values : [ ObjectId() , ObjectId() , ObjectId() , ObjectId() , ObjectId() , ObjectId() ] , keyfield : "_id" } ,
+    { name : "oid_other" , values : [ ObjectId() , ObjectId() , ObjectId() , ObjectId() , ObjectId() , ObjectId() ] , keyfield : "o" } ,
     ]
 
 s = new ShardingTest( "key_many" , 2 );
@@ -20,7 +24,18 @@ seconday = s.getOther( primary ).getDB( "test" );
 
 function makeObjectDotted( v ){
     var o = {};
-    o[curT.keyfield] = v;
+    if (curT.compound){
+        var prefix = curT.keyfield + '.';
+        if (typeof(v) == 'object'){
+            for (key in v)
+                o[prefix + key] = v[key];
+        } else {
+            for (key in curT.values[0])
+                o[prefix + key] = v;
+        }
+    } else {
+        o[curT.keyfield] = v;
+    }
     return o;
 }
 
@@ -39,6 +54,15 @@ function makeObject( v ){
     return o;
 }
 
+function makeInQuery(){
+    if (curT.compound){
+        // cheating a bit...
+        return {'o.a': {$in: [1,2]}};
+    } else {
+        return makeObjectDotted({$in: curT.values});
+    }
+}
+
 function getKey( o ){
     var keys = curT.keyfield.split('.');
     for(var i=0; i<keys.length; i++){
@@ -85,7 +109,20 @@ for ( var i=0; i<types.length; i++ ){
     
     assert.eq( 6 , c.find().sort( makeObjectDotted( 1 ) ).count() , curT.name + " total count with count()" );
 
+    assert.eq( 2 , c.find({$or:[makeObjectDotted(curT.values[2]), makeObjectDotted(curT.values[4])]}).count() , curT.name + " $or count()" );
+    assert.eq( 2 , c.find({$or:[makeObjectDotted(curT.values[2]), makeObjectDotted(curT.values[4])]}).itcount() , curT.name + " $or itcount()" );
+    assert.eq( 4 , c.find({$nor:[makeObjectDotted(curT.values[2]), makeObjectDotted(curT.values[4])]}).count() , curT.name + " $nor count()" );
+    assert.eq( 4 , c.find({$nor:[makeObjectDotted(curT.values[2]), makeObjectDotted(curT.values[4])]}).itcount() , curT.name + " $nor itcount()" );
+
+    var stats = c.stats();
+    printjson( stats )
+    assert.eq( 6 , stats.count , curT.name + " total count with stats()" );
+    var count = 0;
+    for (shard in stats.shards) count += stats.shards[shard].count;
+    assert.eq( 6 , count , curT.name + " total count with stats() sum" );
+
     assert.eq( curT.values , c.find().sort( makeObjectDotted( 1 ) ).toArray().map( getKey ) , curT.name + " sort 1" );
+    assert.eq( curT.values , c.find(makeInQuery()).sort( makeObjectDotted( 1 ) ).toArray().map( getKey ) , curT.name + " sort 1 - $in" );
     assert.eq( curT.values.reverse() , c.find().sort( makeObjectDotted( -1 ) ).toArray().map( getKey ) , curT.name + " sort 2" );
 
 
