@@ -17,17 +17,23 @@ for ( var i=1; i<20; i++ ){
     c.save( { a : i } );
 }
 c.save( { a : 99 } );
+db.getLastError();
 
-assert.eq( s.admin.runCommand( { splitvalue : "test.foo" , find : { a : 1 } } ).middle.a , 1 , "splitvalue 1" );
-assert.eq( s.admin.runCommand( { splitvalue : "test.foo" , find : { a : 3 } } ).middle.a , 1 , "splitvalue 2" );
+function checkSplit( f, want , num ){
+    x = s.admin.runCommand( { splitvalue : "test.foo" , find : { a : f } } );
+    assert.eq( want, x.middle ? x.middle.a : null , "splitvalue " + num + " " + tojson( x ) );
+}
+
+checkSplit( 1 , 1 , "1" )
+checkSplit( 3 , 1 , "2" )
 
 s.adminCommand( { split : "test.foo" , find : { a : 1 } } );
-assert.eq( s.admin.runCommand( { splitvalue : "test.foo" , find : { a : 3 } } ).middle.a , 99 , "splitvalue 3" );
+checkSplit( 3 , 99 , "3" )
 s.adminCommand( { split : "test.foo" , find : { a : 99 } } );
 
 assert.eq( s.config.chunks.count() , 3 );
 s.printChunks();
 
-assert.eq( s.admin.runCommand( { splitvalue : "test.foo" , find : { a : 50 } } ).middle.a , 10 , "splitvalue 4 " );
+checkSplit( 50 , 10 , "4" )
 
 s.stop();
