@@ -22,14 +22,19 @@
    mostly around shard management and checking
  */
 
-#include "stdafx.h"
+#include "pch.h"
 #include "util.h"
 
 using namespace std;
 
 namespace mongo {
 
-    void checkShardVersion( DBClientBase & conn , const string& ns , bool authoritative ){
+    bool checkShardVersion( DBClientBase & conn , const string& ns , bool authoritative , int tryNumber ){
+        // no-op in mongod
+        return false;
+    }
+    
+    void resetShardVersion( DBClientBase * conn ){
         // no-op in mongod
     }
 
