@@ -42,15 +42,15 @@ namespace mongo {
          */
         BSONObj globalMax() const { return gMax; }
 
-        bool isGlobalMin( const BSONObj& k ){
+        bool isGlobalMin( const BSONObj& k ) const{
             return k.woCompare( globalMin() ) == 0;
         }
 
-        bool isGlobalMax( const BSONObj& k ){
+        bool isGlobalMax( const BSONObj& k ) const{
             return k.woCompare( globalMax() ) == 0;
         }
         
-        bool isGlobal( const BSONObj& k ){
+        bool isGlobal( const BSONObj& k ) const{
             return isGlobalMin( k ) || isGlobalMax( k );
         }
 
@@ -59,46 +59,16 @@ namespace mongo {
            l == r 0
            l > r positive
          */
-        int compare( const BSONObj& l , const BSONObj& r );
+        int compare( const BSONObj& l , const BSONObj& r ) const;
         
         /**
            @return whether or not obj has all fields in this shard key pattern
 		   e.g. 
 		     ShardKey({num:1}).hasShardKey({ name:"joe", num:3 }) is true
          */
-        bool hasShardKey( const BSONObj& obj );
+        bool hasShardKey( const BSONObj& obj ) const;
         
-        /**
-           returns a query that filters results only for the range desired, i.e. returns 
-             { "field" : { $gte: keyval(min), $lt: keyval(max) } }
-        */
-        void getFilter( BSONObjBuilder& b , const BSONObj& min, const BSONObj& max );
-        
-        /** @return true if shard s is relevant for query q.
-
-            Example:
-              q:     { x : 3 }
-              *this: { x : 1 }
-              s:     x:2..x:7
-               -> true
-         */
-        bool relevantForQuery( const BSONObj& q , Chunk * s );
-        
-        /**
-           Returns if the given sort pattern can be ordered by the shard key pattern.
-           Example
-            sort:   { ts: -1 }
-            *this:  { ts:1 }
-              -> -1
-
-              @return
-              0 if sort either doesn't have all the fields or has extra fields
-              < 0 if sort is descending
-              > 1 if sort is ascending
-         */
-        int canOrder( const BSONObj& sort );
-
-        BSONObj key() { return pattern; }
+        BSONObj key() const { return pattern; }
 
         string toString() const;
 
@@ -108,21 +78,25 @@ namespace mongo {
             return patternfields.count( key ) > 0;
         }
 
-        operator string() const {
-            return pattern.toString();
-        }
+        /**
+         * @return
+         * true if 'this' is a prefix (not necessarily contained) of 'otherPattern'.
+         */
+        bool isPrefixOf( const BSONObj& otherPattern ) const;
+        
     private:
         BSONObj pattern;
         BSONObj gMin;
         BSONObj gMax;
 
-        /* question: better to have patternfields precomputed or not?  depends on if we use copy contructor often. */
+        /* question: better to have patternfields precomputed or not?  depends on if we use copy constructor often. */
         set<string> patternfields;
-        bool relevant(const BSONObj& query, const BSONObj& L, const BSONObj& R);
     };
 
     inline BSONObj ShardKeyPattern::extractKey(const BSONObj& from) const { 
-        return from.extractFields(pattern);
+        BSONObj k = from.extractFields(pattern);
+        uassert(13334, "Shard Key must be less than 512 bytes", k.objsize() < 512);
+        return k;
     }
 
 } 
