@@ -16,10 +16,12 @@
 
 // strategy_sharded.cpp
 
-#include "stdafx.h"
+#include "pch.h"
 #include "request.h"
 #include "chunk.h"
 #include "cursors.h"
+#include "stats.h"
+
 #include "../client/connpool.h"
 #include "../db/commands.h"
 
@@ -37,28 +39,25 @@ namespace mongo {
             if ( q.ntoreturn == 1 && strstr(q.ns, ".$cmd") )
                 throw UserException( 8010 , "something is wrong, shouldn't see a command here" );
 
-            ChunkManager * info = r.getChunkManager();
+            ChunkManagerPtr info = r.getChunkManager();
             assert( info );
             
             Query query( q.query );
 
-            vector<Chunk*> shards;
-            info->getChunksForQuery( shards , query.getFilter()  );
+            set<Shard> shards;
+            info->getShardsForQuery( shards , query.getFilter()  );
             
             set<ServerAndQuery> servers;
-            map<string,int> serverCounts;
-            for ( vector<Chunk*>::iterator i = shards.begin(); i != shards.end(); i++ ){
-                servers.insert( ServerAndQuery( (*i)->getShard() , (*i)->getFilter() ) );
-                int& num = serverCounts[(*i)->getShard()];
-                num++;
+            for ( set<Shard>::iterator i = shards.begin(); i != shards.end(); i++ ){
+                servers.insert( ServerAndQuery( i->getConnString() , BSONObj() ) ); 
             }
             
             if ( logLevel > 4 ){
                 StringBuilder ss;
-                ss << " shard query servers: " << servers.size() << "\n";
+                ss << " shard query servers: " << servers.size() << '\n';
                 for ( set<ServerAndQuery>::iterator i = servers.begin(); i!=servers.end(); i++ ){
                     const ServerAndQuery& s = *i;
-                    ss << "       " << s.toString() << "\n";
+                    ss << "       " << s.toString() << '\n';
                 }
                 log() << ss.str();
             }
@@ -68,33 +67,33 @@ namespace mongo {
             BSONObj sort = query.getSort();
             
             if ( sort.isEmpty() ){
-                // 1. no sort, can just hit them in serial
                 cursor = new SerialServerClusteredCursor( servers , q );
             }
             else {
-                int shardKeyOrder = info->getShardKey().canOrder( sort );
-                if ( shardKeyOrder ){
-                    // 2. sort on shard key, can do in serial intelligently
-                    set<ServerAndQuery> buckets;
-                    for ( vector<Chunk*>::iterator i = shards.begin(); i != shards.end(); i++ ){
-                        Chunk * s = *i;
-                        buckets.insert( ServerAndQuery( s->getShard() , s->getFilter() , s->getMin() ) );
-                    }
-                    cursor = new SerialServerClusteredCursor( buckets , q , shardKeyOrder );
-                }
-                else {
-                    // 3. sort on non-sharded key, pull back a portion from each server and iterate slowly
-                    cursor = new ParallelSortClusteredCursor( servers , q , sort );
-                }
+                cursor = new ParallelSortClusteredCursor( servers , q , sort );
             }
 
             assert( cursor );
-            
-            log(5) << "   cursor type: " << cursor->type() << endl;
 
-            ShardedClientCursor * cc = new ShardedClientCursor( q , cursor );
+            try {
+                cursor->init();
+
+                log(5) << "   cursor type: " << cursor->type() << endl;
+                shardedCursorTypes.hit( cursor->type() );
+            
+                if ( query.isExplain() ){
+                    BSONObj explain = cursor->explain();
+                    replyToQuery( 0 , r.p() , r.m() , explain );
+                    delete( cursor );
+                    return;
+                }
+            } catch(...) {
+                delete cursor;
+                throw;
+            }
+
+            ShardedClientCursorPtr cc (new ShardedClientCursor( q , cursor ));
             if ( ! cc->sendNextBatch( r ) ){
-                delete( cursor );
                 return;
             }
             log(6) << "storing cursor : " << cc->getId() << endl;
@@ -107,23 +106,24 @@ namespace mongo {
 
             log(6) << "want cursor : " << id << endl;
 
-            ShardedClientCursor * cursor = cursorCache.get( id );
+            ShardedClientCursorPtr cursor = cursorCache.get( id );
             if ( ! cursor ){
                 log(6) << "\t invalid cursor :(" << endl;
-                replyToQuery( QueryResult::ResultFlag_CursorNotFound , r.p() , r.m() , 0 , 0 , 0 );
+                replyToQuery( ResultFlag_CursorNotFound , r.p() , r.m() , 0 , 0 , 0 );
                 return;
             }
             
             if ( cursor->sendNextBatch( r , ntoreturn ) ){
-                log(6) << "\t cursor finished: " << id << endl;
+                // still more data
+                cursor->accessed();
                 return;
             }
             
-            delete( cursor );
+            // we've exhausted the cursor
             cursorCache.remove( id );
         }
         
-        void _insert( Request& r , DbMessage& d, ChunkManager* manager ){
+        void _insert( Request& r , DbMessage& d, ChunkManagerPtr manager ){
             
             while ( d.moreJSObjs() ){
                 BSONObj o = d.nextJsObj();
@@ -146,15 +146,32 @@ namespace mongo {
                     
                 }
                 
-                Chunk& c = manager->findChunk( o );
-                log(4) << "  server:" << c.getShard() << " " << o << endl;
-                insert( c.getShard() , r.getns() , o );
-                
-                c.splitIfShould( o.objsize() );
+                bool gotThrough = false;
+                for ( int i=0; i<10; i++ ){
+                    try {
+                        ChunkPtr c = manager->findChunk( o );
+                        log(4) << "  server:" << c->getShard().toString() << " " << o << endl;
+                        insert( c->getShard() , r.getns() , o );
+                        
+                        r.gotInsert();
+                        c->splitIfShould( o.objsize() );
+                        gotThrough = true;
+                        break;
+                    }
+                    catch ( StaleConfigException& ){
+                        log(1) << "retrying insert because of StaleConfigException: " << o << endl;
+                        r.reset();
+                        manager = r.getChunkManager();
+                    }
+                    sleepmillis( i * 200 );
+                }
+
+                assert( gotThrough );
+
             }            
         }
 
-        void _update( Request& r , DbMessage& d, ChunkManager* manager ){
+        void _update( Request& r , DbMessage& d, ChunkManagerPtr manager ){
             int flags = d.pullInt();
             
             BSONObj query = d.nextJsObj();
@@ -166,8 +183,7 @@ namespace mongo {
             bool upsert = flags & UpdateOption_Upsert;
             bool multi = flags & UpdateOption_Multi;
 
-            if ( multi )
-                uassert( 10202 ,  "can't mix multi and upsert and sharding" , ! upsert );
+            uassert( 10202 ,  "can't mix multi and upsert and sharding" , ! ( upsert && multi ) );
 
             if ( upsert && !(manager->hasShardKey(toupdate) ||
                              (toupdate.firstElement().fieldName()[0] == '$' && manager->hasShardKey(query))))
@@ -179,8 +195,8 @@ namespace mongo {
             if ( ! manager->hasShardKey( query ) ){
                 if ( multi ){
                 }
-                else if ( query.nFields() != 1 || strcmp( query.firstElement().fieldName() , "_id" ) ){
-                    throw UserException( 8013 , "can't do update with query that doesn't have the shard key" );
+                else if ( strcmp( query.firstElement().fieldName() , "_id" ) || query.nFields() != 1 ){
+                    throw UserException( 8013 , "can't do non-multi update with query that doesn't have the shard key" );
                 }
                 else {
                     save = true;
@@ -191,34 +207,57 @@ namespace mongo {
             
             if ( ! save ){
                 if ( toupdate.firstElement().fieldName()[0] == '$' ){
-                    // TODO: check for $set, etc.. on shard key
-                }
-                else if ( manager->hasShardKey( toupdate ) && manager->getShardKey().compare( query , toupdate ) ){
-                    throw UserException( 8014 , "change would move shards!" );
+                    BSONObjIterator ops(toupdate);
+                    while(ops.more()){
+                        BSONElement op(ops.next());
+                        if (op.type() != Object)
+                            continue;
+                        BSONObjIterator fields(op.embeddedObject());
+                        while(fields.more()){
+                            const string field = fields.next().fieldName();
+                            uassert(13123, "Can't modify shard key's value", ! manager->getShardKey().partOfShardKey(field));
+                        }
+                    }
+                } else if ( manager->hasShardKey( toupdate ) ){
+                    uassert( 8014, "change would move shards!", manager->getShardKey().compare( query , toupdate ) == 0 );
+                } else {
+                    uasserted(12376, "shard key must be in update object");
                 }
             }
             
             if ( multi ){
-                vector<Chunk*> chunks;
-                manager->getChunksForQuery( chunks , chunkFinder );
-                set<string> seen;
-                for ( vector<Chunk*>::iterator i=chunks.begin(); i!=chunks.end(); i++){
-                    Chunk * c = *i;
-                    if ( seen.count( c->getShard() ) )
-                        continue;
-                    doWrite( dbUpdate , r , c->getShard() );
-                    seen.insert( c->getShard() );
+                set<Shard> shards;
+                manager->getShardsForQuery( shards , chunkFinder );
+                int * x = (int*)(r.d().afterNS());
+                x[0] |= UpdateOption_Broadcast;
+                for ( set<Shard>::iterator i=shards.begin(); i!=shards.end(); i++){
+                    doWrite( dbUpdate , r , *i , false );
                 }
             }
             else {
-                Chunk& c = manager->findChunk( chunkFinder );
-                doWrite( dbUpdate , r , c.getShard() );
-                c.splitIfShould( d.msg().data->dataLen() );
+                int left = 5;
+                while ( true ){
+                    try {
+                        ChunkPtr c = manager->findChunk( chunkFinder );
+                        doWrite( dbUpdate , r , c->getShard() );
+                        c->splitIfShould( d.msg().header()->dataLen() );
+                        break;
+                    }
+                    catch ( StaleConfigException& e ){
+                        if ( left <= 0 )
+                            throw e;
+                        left--;
+                        log() << "update failed b/c of StaleConfigException, retrying " 
+                              << " left:" << left << " ns: " << r.getns() << " query: " << query << endl;
+                        r.reset( false );
+                        manager = r.getChunkManager();
+                    }
+                }
             }
 
         }
         
-        void _delete( Request& r , DbMessage& d, ChunkManager* manager ){
+        void _delete( Request& r , DbMessage& d, ChunkManagerPtr manager ){
 
             int flags = d.pullInt();
             bool justOne = flags & 1;
@@ -226,24 +265,38 @@ namespace mongo {
             uassert( 10203 ,  "bad delete message" , d.moreJSObjs() );
             BSONObj pattern = d.nextJsObj();
 
-            vector<Chunk*> chunks;
-            manager->getChunksForQuery( chunks , pattern );
-            cout << "delete : " << pattern << " \t " << chunks.size() << " justOne: " << justOne << endl;
-            if ( chunks.size() == 1 ){
-                doWrite( dbDelete , r , chunks[0]->getShard() );
-                return;
+            set<Shard> shards;
+            int left = 5;
+            
+            while ( true ){
+                try {
+                    manager->getShardsForQuery( shards , pattern );
+                    log(2) << "delete : " << pattern << " \t " << shards.size() << " justOne: " << justOne << endl;
+                    if ( shards.size() == 1 ){
+                        doWrite( dbDelete , r , *shards.begin() );
+                        return;
+                    }
+                    break;
+                }
+                catch ( StaleConfigException& e ){
+                    if ( left <= 0 )
+                        throw e;
+                    left--;
+                    log() << "delete failed b/c of StaleConfigException, retrying " 
+                          << " left:" << left << " ns: " << r.getns() << " patt: " << pattern << endl;
+                    r.reset( false );
+                    shards.clear();
+                    manager = r.getChunkManager();
+                }
             }
             
             if ( justOne && ! pattern.hasField( "_id" ) )
                 throw UserException( 8015 , "can only delete with a non-shard key pattern if can delete as many as we find" );
             
-            set<string> seen;
-            for ( vector<Chunk*>::iterator i=chunks.begin(); i!=chunks.end(); i++){
-                Chunk * c = *i;
-                if ( seen.count( c->getShard() ) )
-                    continue;
-                seen.insert( c->getShard() );
-                doWrite( dbDelete , r , c->getShard() );
+            for ( set<Shard>::iterator i=shards.begin(); i!=shards.end(); i++){
+                int * x = (int*)(r.d().afterNS());
+                x[0] |= RemoveOption_Broadcast;
+                doWrite( dbDelete , r , *i , false );
             }
         }
         
@@ -252,7 +305,7 @@ namespace mongo {
             log(3) << "write: " << ns << endl;
             
             DbMessage& d = r.d();
-            ChunkManager * info = r.getChunkManager();
+            ChunkManagerPtr info = r.getChunkManager();
             assert( info );
             
             if ( op == dbInsert ){
