@@ -17,7 +17,7 @@
 
 #pragma once
 
-#include "../stdafx.h"
+#include "../pch.h"
 #include "../db/jsobj.h"
 
 extern const char * jsconcatcode; // TODO: change name to mongoJSCode
@@ -162,5 +162,7 @@ namespace mongo {
         static void ( *_connectCallback )( DBClientWithCommands & );
     };
 
+    bool hasJSReturn( const string& s );
+
     extern ScriptEngine * globalScriptEngine;
 }
