@@ -20,6 +20,7 @@
 #include "v8_db.h"
 #include "engine.h"
 #include "util/base64.h"
+#include "util/text.h"
 #include "../client/syncclusterconnection.h"
 #include <iostream>
 
@@ -28,12 +29,11 @@ using namespace v8;
 
 namespace mongo {
 
-#define CONN_STRING (v8::String::New( "_conn" ))
-
 #define DDD(x)
 
     v8::Handle<v8::FunctionTemplate> getMongoFunctionTemplate( bool local ){
         v8::Local<v8::FunctionTemplate> mongo = FunctionTemplate::New( local ? mongoConsLocal : mongoConsExternal );
+        mongo->InstanceTemplate()->SetInternalFieldCount( 1 );
         
         v8::Local<v8::Template> proto = mongo->PrototypeTemplate();
 
@@ -43,9 +43,13 @@ namespace mongo {
         proto->Set( v8::String::New( "update" ) , FunctionTemplate::New( mongoUpdate ) );
 
         Local<FunctionTemplate> ic = FunctionTemplate::New( internalCursorCons );
+        ic->InstanceTemplate()->SetInternalFieldCount( 1 );
         ic->PrototypeTemplate()->Set( v8::String::New("next") , FunctionTemplate::New( internalCursorNext ) );
         ic->PrototypeTemplate()->Set( v8::String::New("hasNext") , FunctionTemplate::New( internalCursorHasNext ) );
+        ic->PrototypeTemplate()->Set( v8::String::New("objsLeftInBatch") , FunctionTemplate::New( internalCursorObjsLeftInBatch ) );
         proto->Set( v8::String::New( "internalCursor" ) , ic );
+        
+
 
         return mongo;
     }
@@ -131,9 +135,10 @@ namespace mongo {
         global->Get( v8::String::New( "Object" ) )->ToObject()->Set( v8::String::New("bsonsize") , FunctionTemplate::New( bsonsize )->GetFunction() );
     }
 
-    void destroyConnection( Persistent<Value> object, void* parameter){
-        // TODO
-        cout << "warning: destroyConnection not implemented" << endl;
+    void destroyConnection( Persistent<Value> self, void* parameter){
+        delete static_cast<DBClientBase*>(parameter);
+        self.Dispose();
+        self.Clear();
     }
 
     Handle<Value> mongoConsExternal(const Arguments& args){
@@ -148,47 +153,22 @@ namespace mongo {
             strcpy( host , "127.0.0.1" );
         }
 
-        DBClientWithCommands * conn = 0;
-        int commas = 0;
-        for ( int i=0; i<255; i++ ){
-            if ( host[i] == ',' )
-                commas++;
-            else if ( host[i] == 0 )
-                break;
-        }
+        string errmsg;
+        ConnectionString cs = ConnectionString::parse( host , errmsg );
+        if ( ! cs.isValid() )
+            return v8::ThrowException( v8::String::New( errmsg.c_str() ) );
         
-        if ( commas == 0 ){
-            DBClientConnection * c = new DBClientConnection( true );
-            string errmsg;
-            if ( ! c->connect( host , errmsg ) ){
-                delete c;
-                string x = "couldn't connect: ";
-                x += errmsg;
-                return v8::ThrowException( v8::String::New( x.c_str() ) );
-            }
-            conn = c;
-        }
-        else if ( commas == 1 ){
-            DBClientPaired * c = new DBClientPaired();
-            if ( ! c->connect( host ) ){
-                delete c;
-                return v8::ThrowException( v8::String::New( "couldn't connect to pair" ) );
-            }
-            conn = c;
-        }
-        else if ( commas == 2 ){
-            conn = new SyncClusterConnection( host );
-        }
-        else {
-            return v8::ThrowException( v8::String::New( "too many commas" ) );
-        }
-                
-        Persistent<v8::Object> self = Persistent<v8::Object>::New( args.This() );
+        
+        DBClientWithCommands * conn = cs.connect( errmsg );
+        if ( ! conn )
+            return v8::ThrowException( v8::String::New( errmsg.c_str() ) );
+        
+        Persistent<v8::Object> self = Persistent<v8::Object>::New( args.Holder() );
         self.MakeWeak( conn , destroyConnection );
 
         ScriptEngine::runConnectCallback( *conn );
-        // NOTE I don't believe the conn object will ever be freed.
-        args.This()->Set( CONN_STRING , External::New( conn ) );
+
+        args.This()->SetInternalField( 0 , External::New( conn ) );
         args.This()->Set( v8::String::New( "slaveOk" ) , Boolean::New( false ) );
         args.This()->Set( v8::String::New( "host" ) , v8::String::New( host ) );
     
@@ -206,7 +186,7 @@ namespace mongo {
         self.MakeWeak( conn , destroyConnection );
 
         // NOTE I don't believe the conn object will ever be freed.
-        args.This()->Set( CONN_STRING , External::New( conn ) );
+        args.This()->SetInternalField( 0 , External::New( conn ) );
         args.This()->Set( v8::String::New( "slaveOk" ) , Boolean::New( false ) );
         args.This()->Set( v8::String::New( "host" ) , v8::String::New( "EMBEDDED" ) );
         
@@ -223,7 +203,7 @@ namespace mongo {
 #endif
 
     DBClientBase * getConnection( const Arguments& args ){
-        Local<External> c = External::Cast( *(args.This()->Get( CONN_STRING )) );
+        Local<External> c = External::Cast( *(args.This()->GetInternalField( 0 )) );
         DBClientBase * conn = (DBClientBase*)(c->Value());
         assert( conn );
         return conn;
@@ -231,6 +211,12 @@ namespace mongo {
 
     // ---- real methods
 
+    void destroyCursor( Persistent<Value> self, void* parameter){
+        delete static_cast<mongo::DBClientCursor*>(parameter);
+        self.Dispose();
+        self.Clear();
+    }
+
     /**
        0 - namespace
        1 - query
@@ -239,6 +225,8 @@ namespace mongo {
        4 - skip
     */
     Handle<Value> mongoFind(const Arguments& args){
+        HandleScope handle_scope;
+
         jsassert( args.Length() == 6 , "find needs 6 args" );
         jsassert( args[1]->IsObject() , "needs to be an object" );
         DBClientBase * conn = getConnection( args );
@@ -268,11 +256,12 @@ namespace mongo {
             }
             v8::Function * cons = (v8::Function*)( *( mongo->Get( v8::String::New( "internalCursor" ) ) ) );
             assert( cons );
-            Local<v8::Object> c = cons->NewInstance();
-        
-            // NOTE I don't believe the cursor object will ever be freed.
-            c->Set( v8::String::New( "cursor" ) , External::New( cursor.release() ) );
-            return c;
+            
+            Persistent<v8::Object> c = Persistent<v8::Object>::New( cons->NewInstance() );
+            c.MakeWeak( cursor.get() , destroyCursor );
+            
+            c->SetInternalField( 0 , External::New( cursor.release() ) );
+            return handle_scope.Close(c);
         }
         catch ( ... ){
             return v8::ThrowException( v8::String::New( "socket error on query" ) );        
@@ -308,7 +297,7 @@ namespace mongo {
     }
 
     v8::Handle<v8::Value> mongoRemove(const v8::Arguments& args){
-        jsassert( args.Length() == 2 , "remove needs 2 args" );
+        jsassert( args.Length() == 2 || args.Length() == 3 , "remove needs 2 args" );
         jsassert( args[1]->IsObject() , "have to remove an object template" );
 
         DBClientBase * conn = getConnection( args );
@@ -317,10 +306,15 @@ namespace mongo {
         v8::Handle<v8::Object> in = args[1]->ToObject();
         BSONObj o = v8ToMongo( in );
     
+        bool justOne = false;
+        if ( args.Length() > 2 ){
+            justOne = args[2]->BooleanValue();
+        }
+
         DDD( "want to remove : " << o.jsonString() );
         try {
             v8::Unlocker u;
-            conn->remove( ns , o );
+            conn->remove( ns , o , justOne );
         }
         catch ( ... ){
             return v8::ThrowException( v8::String::New( "socket error on remove" ) );
@@ -362,7 +356,8 @@ namespace mongo {
     // --- cursor ---
 
     mongo::DBClientCursor * getCursor( const Arguments& args ){
-        Local<External> c = External::Cast( *(args.This()->Get( v8::String::New( "cursor" ) ) ) );
+        Local<External> c = External::Cast( *(args.This()->GetInternalField( 0 ) ) );
+
         mongo::DBClientCursor * cursor = (mongo::DBClientCursor*)(c->Value());
         return cursor;
     }
@@ -395,6 +390,18 @@ namespace mongo {
         return Boolean::New( ret );
     }
 
+    v8::Handle<v8::Value> internalCursorObjsLeftInBatch(const v8::Arguments& args){
+        mongo::DBClientCursor * cursor = getCursor( args );
+        if ( ! cursor )
+            return v8::Number::New( (double) 0 );
+        int ret;
+        {
+            v8::Unlocker u;
+            ret = cursor->objsLeftInBatch();
+        }
+        return v8::Number::New( (double) ret );
+    }
+
 
     // --- DB ----
 
@@ -623,17 +630,17 @@ namespace mongo {
         v8::String::Utf8Value data( it->Get( v8::String::New( "data" ) ) );
         
         stringstream ss;
-        ss << "BinData( type: " << type << ", base64: \"";
+        ss << "BinData(" << type << ",\"";
         base64::encode( ss, *data, len );
-        ss << "\" )";
+        ss << "\")";
         string ret = ss.str();
         return v8::String::New( ret.c_str() );
     }
 
     v8::Handle<v8::Value> numberLongInit( const v8::Arguments& args ) {
         
-        if (args.Length() != 1 && args.Length() != 3) {
-            return v8::ThrowException( v8::String::New( "NumberLong needs 1 or 3 arguments" ) );
+        if (args.Length() != 0 && args.Length() != 1 && args.Length() != 3) {
+            return v8::ThrowException( v8::String::New( "NumberLong needs 0, 1 or 3 arguments" ) );
         }
         
         v8::Handle<v8::Object> it = args.This();
@@ -642,9 +649,33 @@ namespace mongo {
             v8::Function* f = getNamedCons( "NumberLong" );
             it = f->NewInstance();
         }
-        
-        it->Set( v8::String::New( "floatApprox" ) , args[0] );
-        if ( args.Length() == 3 ) {
+
+        if ( args.Length() == 0 ) {
+            it->Set( v8::String::New( "floatApprox" ), v8::Number::New( 0 ) );
+        } else if ( args.Length() == 1 ) {
+            if ( args[ 0 ]->IsNumber() ) {
+                it->Set( v8::String::New( "floatApprox" ), args[ 0 ] );            
+            } else {
+                v8::String::Utf8Value data( args[ 0 ] );
+                string num = *data;
+                const char *numStr = num.c_str();
+                long long n;
+                try {
+                    n = parseLL( numStr );
+                } catch ( const AssertionException & ) {
+                    return v8::ThrowException( v8::String::New( "could not convert string to long long" ) );
+                }
+                unsigned long long val = n;
+                if ( (long long)val == (long long)(double)(long long)(val) ) {
+                    it->Set( v8::String::New( "floatApprox" ), v8::Number::New( (double)(long long)( val ) ) );
+                } else {
+                    it->Set( v8::String::New( "floatApprox" ), v8::Number::New( (double)(long long)( val ) ) );
+                    it->Set( v8::String::New( "top" ), v8::Integer::New( val >> 32 ) );
+                    it->Set( v8::String::New( "bottom" ), v8::Integer::New( (unsigned long)(val & 0x00000000ffffffff) ) );
+                }                
+            }
+        } else {
+            it->Set( v8::String::New( "floatApprox" ) , args[0] );
             it->Set( v8::String::New( "top" ) , args[1] );
             it->Set( v8::String::New( "bottom" ) , args[2] );
         }
@@ -687,10 +718,15 @@ namespace mongo {
         
         v8::Handle<v8::Object> it = args.This();
         
-        long long val = numberLongVal( it );
-        
         stringstream ss;
-        ss << val;
+        long long val = numberLongVal( it );
+        const long long limit = 2LL << 30;
+
+        if ( val <= -limit || limit <= val )
+            ss << "NumberLong(\"" << val << "\")";
+        else
+            ss << "NumberLong(" << val << ")";
+
         string ret = ss.str();
         return v8::String::New( ret.c_str() );
     }
