@@ -15,9 +15,15 @@
  *    limitations under the License.
  */
 
-
+#include "pch.h"
 #include <stdio.h>
 
+#if defined(_WIN32)
+# if defined(USE_READLINE)
+# define USE_READLINE_STATIC
+# endif
+#endif
+
 #ifdef USE_READLINE
 #include <readline/readline.h>
 #include <readline/history.h>
@@ -30,18 +36,87 @@ jmp_buf jbuf;
 #include "../util/unittest.h"
 #include "../db/cmdline.h"
 #include "utils.h"
+#include "../util/password.h"
+#include "../util/version.h"
+#include "../util/goodies.h"
 
 using namespace std;
 using namespace boost::filesystem;
 
+using mongo::BSONObj;
+using mongo::BSONObjBuilder;
+using mongo::BSONObjIterator;
+using mongo::BSONElement;
+
 string historyFile;
 bool gotInterrupted = 0;
 bool inMultiLine = 0;
+static volatile bool atPrompt = false; // can eval before getting to prompt
+bool autoKillOp = false;
 
-#if defined(USE_READLINE) && !defined(__freebsd__) && !defined(_WIN32)
+
+#if defined(USE_READLINE) && !defined(__freebsd__) && !defined(__openbsd__) && !defined(_WIN32)
 #define CTRLC_HANDLE
 #endif
 
+mongo::Scope * shellMainScope;
+
+void generateCompletions( const string& prefix , vector<string>& all ){
+    if ( prefix.find( '"' ) != string::npos )
+        return;
+    shellMainScope->exec( "shellAutocomplete( \"" + prefix + "\" );" , "autocomplete help" , false , true , false );
+    
+    BSONObjBuilder b;
+    shellMainScope->append( b , "" , "__autocomplete__" );
+    BSONObj res = b.obj();
+    BSONObj arr = res.firstElement().Obj();
+
+    BSONObjIterator i(arr);
+    while ( i.more() ){
+        BSONElement e = i.next();
+        all.push_back( e.String() );
+    }
+
+}
+
+#ifdef USE_READLINE
+static char** completionHook(const char* text , int start ,int end ){
+    static map<string,string> m;
+    
+    vector<string> all;
+    
+    if ( start == 0 ){
+        generateCompletions( string(text,end) , all );
+    }
+    
+    if ( all.size() == 0 ){
+        rl_bind_key('\t',0);
+        return 0;
+    }
+    
+    string longest = all[0];
+    for ( vector<string>::iterator i=all.begin(); i!=all.end(); ++i ){
+        string s = *i;
+        for ( unsigned j=0; j<s.size(); j++ ){
+            if ( longest[j] == s[j] )
+                continue;
+            longest = longest.substr(0,j);
+            break;
+        }
+    }
+    
+    char ** matches = (char**)malloc( sizeof(char*) * (all.size()+2) );
+    unsigned x=0;
+    matches[x++] = strdup( longest.c_str() );
+    for ( unsigned i=0; i<all.size(); i++ ){
+        matches[x++] = strdup( all[i].c_str() );
+    }
+    matches[x++] = 0;
+
+    return matches;
+}
+#endif
+
 void shellHistoryInit(){
 #ifdef USE_READLINE
     stringstream ss;
@@ -53,9 +128,11 @@ void shellHistoryInit(){
 
     using_history();
     read_history( historyFile.c_str() );
-
+    
+    rl_attempted_completion_function = completionHook;
+        
 #else
-    cout << "type \"exit\" to exit" << endl;
+    //cout << "type \"exit\" to exit" << endl;
 #endif
 }
 void shellHistoryDone(){
@@ -64,9 +141,16 @@ void shellHistoryDone(){
 #endif
 }
 void shellHistoryAdd( const char * line ){
-    if ( strlen(line) == 0 )
-        return;
 #ifdef USE_READLINE
+    if ( line[0] == '\0' )
+        return;
+
+    // dont record duplicate lines
+    static string lastLine;
+    if (lastLine == line)
+        return;
+    lastLine = line;
+
     if ((strstr(line, ".auth")) == NULL)
         add_history( line );
 #endif
@@ -82,6 +166,22 @@ void intr( int sig ){
 void killOps() {
     if ( mongo::shellUtils::_nokillop || mongo::shellUtils::_allMyUris.size() == 0 )
         return;
+
+    if ( atPrompt )
+        return;
+
+    if ( !autoKillOp ) {
+        cout << endl << "do you want to kill the current op on the server? (y/n): ";
+        cout.flush();
+
+        char yn;
+        cin >> yn;
+
+        if (yn != 'y' && yn != 'Y')
+            return;
+    }
+
+
     vector< string > uris;
     for( map< const void*, string >::iterator i = mongo::shellUtils::_allMyUris.begin(); i != mongo::shellUtils::_allMyUris.end(); ++i )
         uris.push_back( i->second );
@@ -95,6 +195,7 @@ void killOps() {
 }
 
 void quitNicely( int sig ){
+    mongo::goingAway = true;
     if ( sig == SIGINT && inMultiLine ){
         gotInterrupted = 1;
         return;
@@ -105,14 +206,28 @@ void quitNicely( int sig ){
     shellHistoryDone();
     exit(0);
 }
+#else
+void quitNicely( int sig ){
+    mongo::goingAway = true;
+    //killOps();
+    shellHistoryDone();
+    exit(0);
+}
 #endif
 
 char * shellReadline( const char * prompt , int handlesigint = 0 ){
+    atPrompt = true;
 #ifdef USE_READLINE
 
+    rl_bind_key('\t',rl_complete);
+
+
 #ifdef CTRLC_HANDLE
-    if ( ! handlesigint )
-        return readline( prompt );
+    if ( ! handlesigint ){
+        char* ret = readline( prompt );
+        atPrompt = false;
+        return ret;
+    }
     if ( setjmp( jbuf ) ){
         gotInterrupted = 1;
         sigrelse(SIGINT);
@@ -124,13 +239,16 @@ char * shellReadline( const char * prompt , int handlesigint = 0 ){
 
     char * ret = readline( prompt );
     signal( SIGINT , quitNicely );
+    atPrompt = false;
     return ret;
 #else
-    printf("%s", prompt);
+    printf("%s", prompt); cout.flush();
     char * buf = new char[1024];
     char * l = fgets( buf , 1024 , stdin );
     int len = strlen( buf );
-    buf[len-1] = 0;
+    if ( len )
+        buf[len-1] = 0;
+    atPrompt = false;
     return l;
 #endif
 }
@@ -173,8 +291,8 @@ string fixHost( string url , string host , string port ){
             if ( url.find( "." ) != string::npos )
                 return url + "/test";
 
-            if ( url.find( ":" ) != string::npos &&
-                 isdigit( url[url.find(":")+1] ) )
+            if ( url.rfind( ":" ) != string::npos &&
+                 isdigit( url[url.rfind(":")+1] ) )
                 return url + "/test";
         }
         return url;
@@ -191,6 +309,10 @@ string fixHost( string url , string host , string port ){
     string newurl = host;
     if ( port.size() > 0 )
         newurl += ":" + port;
+    else if (host.find(':') == string::npos){
+        // need to add port with IPv6 addresses
+        newurl += ":27017";
+    }
 
     newurl += "/" + url;
 
@@ -283,7 +405,12 @@ bool fileExists( string file ){
     }
 }
 
+namespace mongo {
+    extern bool isShell;
+}
+
 int _main(int argc, char* argv[]) {
+    mongo::isShell = true;
     setupSignals();
 
     mongo::shellUtils::RecordMyLocation( argv[ 0 ] );
@@ -314,15 +441,18 @@ int _main(int argc, char* argv[]) {
         ("host", po::value<string>(&dbhost), "server to connect to")
         ("eval", po::value<string>(&script), "evaluate javascript")
         ("username,u", po::value<string>(&username), "username for authentication")
-        ("password,p", po::value<string>(&password), "password for authentication")
+        ("password,p", new mongo::PasswordValue(&password),
+         "password for authentication")
         ("help,h", "show this usage information")
         ("version", "show version information")
+        ("ipv6", "enable IPv6 support (disabled by default)")
         ;
 
     hidden_options.add_options()
         ("dbaddress", po::value<string>(), "dbaddress")
         ("files", po::value< vector<string> >(), "files")
         ("nokillop", "nokillop") // for testing, kill op will also be disabled automatically if the tests starts a mongo program
+        ("autokillop", "autokillop") // for testing, will kill op without prompting
         ;
 
     positional_options.add("dbaddress", 1);
@@ -350,6 +480,16 @@ int _main(int argc, char* argv[]) {
         return mongo::EXIT_BADOPTIONS;
     }
 
+    // hide password from ps output
+    for (int i=0; i < (argc-1); ++i){
+        if (!strcmp(argv[i], "-p") || !strcmp(argv[i], "--password")){
+            char* arg = argv[i+1];
+            while (*arg){
+                *arg++ = 'x';
+            }
+        }
+    }
+
     if (params.count("shell")) {
         runShell = true;
     }
@@ -373,6 +513,9 @@ int _main(int argc, char* argv[]) {
     if (params.count("nokillop")) {
         mongo::shellUtils::_nokillop = true;
     }
+    if (params.count("autokillop")) {
+        autoKillOp = true;
+    }
     
     /* This is a bit confusing, here are the rules:
      *
@@ -395,6 +538,9 @@ int _main(int argc, char* argv[]) {
             }
         }
     }
+    if (params.count("ipv6")){
+        mongo::enableIPv6();
+    }
     
     if ( ! mongo::cmdLine.quiet ) 
         cout << "MongoDB shell version: " << mongo::versionString << endl;
@@ -402,7 +548,7 @@ int _main(int argc, char* argv[]) {
     mongo::UnitTest::runTests();
 
     if ( !nodb ) { // connect to db
-        if ( ! mongo::cmdLine.quiet ) cout << "url: " << url << endl;
+        //if ( ! mongo::cmdLine.quiet ) cout << "url: " << url << endl;
         
         stringstream ss;
         if ( mongo::cmdLine.quiet )
@@ -411,6 +557,11 @@ int _main(int argc, char* argv[]) {
         
         mongo::shellUtils::_dbConnect = ss.str();
 
+        if ( params.count( "password" )
+             && ( password.empty() ) ) {
+            password = mongo::askPassword();
+        }
+
         if ( username.size() && password.size() ){
             stringstream ss;
             ss << "if ( ! db.auth( \"" << username << "\" , \"" << password << "\" ) ){ throw 'login failed'; }";
@@ -423,6 +574,10 @@ int _main(int argc, char* argv[]) {
     mongo::ScriptEngine::setup();
     mongo::globalScriptEngine->setScopeInitCallback( mongo::shellUtils::initScope );
     auto_ptr< mongo::Scope > scope( mongo::globalScriptEngine->newScope() );    
+    shellMainScope = scope.get();
+
+    if( runShell )
+        cout << "type \"help\" for help" << endl;
     
     if ( !script.empty() ) {
         mongo::shellUtils::MongoProgramScope s;
@@ -452,8 +607,6 @@ int _main(int argc, char* argv[]) {
 
         shellHistoryInit();
 
-        cout << "type \"help\" for help" << endl;
-
         //v8::Handle<v8::Object> shellHelper = baseContext_->Global()->Get( v8::String::New( "shellHelper" ) )->ToObject();
 
         while ( 1 ){
@@ -510,8 +663,8 @@ int _main(int argc, char* argv[]) {
 
             if ( ! wascmd ){
                 try {
-                    scope->exec( code.c_str() , "(shell)" , false , true , false );
-                    scope->exec( "shellPrintHelper( __lastres__ );" , "(shell2)" , true , true , false );
+                    if ( scope->exec( code.c_str() , "(shell)" , false , true , false ) )
+                        scope->exec( "shellPrintHelper( __lastres__ );" , "(shell2)" , true , true , false );
                 }
                 catch ( std::exception& e ){
                     cout << "error:" << e.what() << endl;
@@ -525,6 +678,7 @@ int _main(int argc, char* argv[]) {
         shellHistoryDone();
     }
 
+    mongo::goingAway = true;
     return 0;
 }
 
