@@ -1,5 +1,6 @@
 const char * jsconcatcode = 
 "__quiet = false;\n"
+ "__magicNoPrint = { __magicNoPrint : 1111 }\n"
  "chatty = function(s){\n"
  "if ( ! __quiet )\n"
  "print( s );}\n"
@@ -9,14 +10,19 @@ const char * jsconcatcode =
  "if ( tojson( a ) == tojson( b ) )\n"
  "return true;\n"
  "return false;}\n"
- "doassert = function( msg ){\n"
- "print( \"assert: \" + msg );\n"
+ "doassert = function (msg) {\n"
+ "if (msg.indexOf(\"assert\") == 0)\n"
+ "print(msg);\n"
+ "else\n"
+ "print(\"assert: \" + msg);\n"
  "throw msg;}\n"
  "assert = function( b , msg ){\n"
  "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
  "if ( b )\n"
  "return;\n"
- "doassert( \"assert failed : \" + msg );}\n"
+ "doassert( msg == undefined ? \"assert failed\" : \"assert failed : \" + msg );}\n"
+ "assert.automsg = function( b ) {\n"
+ "assert( eval( b ), b );}\n"
  "assert._debug = false;\n"
  "assert.eq = function( a , b , msg ){\n"
  "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
@@ -25,11 +31,29 @@ const char * jsconcatcode =
  "if ( ( a != null && b != null ) && friendlyEqual( a , b ) )\n"
  "return;\n"
  "doassert( \"[\" + tojson( a ) + \"] != [\" + tojson( b ) + \"] are not equal : \" + msg );}\n"
+ "assert.eq.automsg = function( a, b ) {\n"
+ "assert.eq( eval( a ), eval( b ), \"[\" + a + \"] != [\" + b + \"]\" );}\n"
  "assert.neq = function( a , b , msg ){\n"
  "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
  "if ( a != b )\n"
  "return;\n"
  "doassert( \"[\" + a + \"] != [\" + b + \"] are equal : \" + msg );}\n"
+ "assert.repeat = function( f, msg, timeout, interval ) {\n"
+ "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
+ "var start = new Date();\n"
+ "timeout = timeout || 30000;\n"
+ "interval = interval || 200;\n"
+ "var last;\n"
+ "while( 1 ) {\n"
+ "if ( typeof( f ) == \"string\" ){\n"
+ "if ( eval( f ) )\n"
+ "return;}\n"
+ "else {\n"
+ "if ( f() )\n"
+ "return;}\n"
+ "if ( ( new Date() ).getTime() - start.getTime() > timeout )\n"
+ "break;\n"
+ "sleep( interval );}}\n"
  "assert.soon = function( f, msg, timeout, interval ) {\n"
  "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
  "var start = new Date();\n"
@@ -53,6 +77,8 @@ const char * jsconcatcode =
  "catch ( e ){\n"
  "return e;}\n"
  "doassert( \"did not throw exception: \" + msg );}\n"
+ "assert.throws.automsg = function( func, params ) {\n"
+ "assert.throws( func, params, func.toString() );}\n"
  "assert.commandWorked = function( res , msg ){\n"
  "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
  "if ( res.ok == 1 )\n"
@@ -78,11 +104,24 @@ const char * jsconcatcode =
  "if ( a > b )\n"
  "return;\n"
  "doassert( a + \" is not greater than \" + b + \" : \" + msg );}\n"
- "assert.close = function( a , b , msg ){\n"
- "var diff = Math.abs( (a-b)/((a+b)/2) );\n"
- "if ( diff < .001 )\n"
+ "assert.lte = function( a , b , msg ){\n"
+ "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
+ "if ( a <= b )\n"
  "return;\n"
- "doassert( a + \" is not close to \" + b + \" diff: \" + diff + \" : \" + msg );}\n"
+ "doassert( a + \" is not less than or eq \" + b + \" : \" + msg );}\n"
+ "assert.gte = function( a , b , msg ){\n"
+ "if ( assert._debug && msg ) print( \"in assert for: \" + msg );\n"
+ "if ( a >= b )\n"
+ "return;\n"
+ "doassert( a + \" is not greater than or eq \" + b + \" : \" + msg );}\n"
+ "assert.close = function( a , b , msg , places ){\n"
+ "if (places === undefined) {\n"
+ "places = 4;}\n"
+ "if (Math.round((a - b) * Math.pow(10, places)) === 0) {\n"
+ "return;}\n"
+ "doassert( a + \" is not equal to \" + b + \" within \" + places +\n"
+ "\" places, diff: \" + (a-b) + \" : \" + msg );\n"
+ "};\n"
  "Object.extend = function( dst , src , deep ){\n"
  "for ( var k in src ){\n"
  "var v = src[k];\n"
@@ -182,6 +221,10 @@ const char * jsconcatcode =
  "if ( !( i in o.__proto__ && o[ i ] === o.__proto__[ i ] ) ) {\n"
  "ret.push( i );}}\n"
  "return ret;}\n"
+ "if ( ! NumberLong.prototype ) {\n"
+ "NumberLong.prototype = {}}\n"
+ "NumberLong.prototype.tojson = function() {\n"
+ "return this.toString();}\n"
  "if ( ! ObjectId.prototype )\n"
  "ObjectId.prototype = {}\n"
  "ObjectId.prototype.toString = function(){\n"
@@ -189,6 +232,10 @@ const char * jsconcatcode =
  "ObjectId.prototype.tojson = function(){\n"
  "return \"ObjectId(\\\"\" + this.str + \"\\\")\";}\n"
  "ObjectId.prototype.isObjectId = true;\n"
+ "ObjectId.prototype.getTimestamp = function(){\n"
+ "return new Date(parseInt(this.toString().slice(0,8), 16)*1000);}\n"
+ "ObjectId.prototype.equals = function( other){\n"
+ "return this.str == other.str;}\n"
  "if ( typeof( DBPointer ) != \"undefined\" ){\n"
  "DBPointer.prototype.fetch = function(){\n"
  "assert( this.ns , \"need a ns\" );\n"
@@ -216,10 +263,14 @@ const char * jsconcatcode =
  "else {\n"
  "print( \"warning: no DBRef\" );}\n"
  "if ( typeof( BinData ) != \"undefined\" ){\n"
- "BinData.prototype.tojson = function(){\n"
- "return \"BinData type: \" + this.type + \" len: \" + this.len;}}\n"
+ "BinData.prototype.tojson = function () {\n"
+ "//return \"BinData type: \" + this.type + \" len: \" + this.len;\n"
+ "return this.toString();}}\n"
  "else {\n"
- "print( \"warning: no BinData\" );}\n"
+ "print( \"warning: no BinData class\" );}\n"
+ "if ( typeof( UUID ) != \"undefined\" ){\n"
+ "UUID.prototype.tojson = function () {\n"
+ "return this.toString();}}\n"
  "if ( typeof _threadInject != \"undefined\" ){\n"
  "print( \"fork() available!\" );\n"
  "Thread = function(){\n"
@@ -291,7 +342,9 @@ const char * jsconcatcode =
  "\"jstests/indexb.js\",\n"
  "\"jstests/profile1.js\",\n"
  "\"jstests/mr3.js\",\n"
- "\"jstests/apitest_db.js\"] );\n"
+ "\"jstests/indexh.js\",\n"
+ "\"jstests/apitest_db.js\",\n"
+ "\"jstests/evalb.js\"] );\n"
  "var serialTestsArr = [ \"jstests/fsync.js\",\n"
  "\"jstests/fsync2.js\" ];\n"
  "var serialTests = makeKeys( serialTestsArr );\n"
@@ -301,8 +354,8 @@ const char * jsconcatcode =
  "var i = 0;\n"
  "files.forEach(\n"
  "function(x) {\n"
- "if ( /_runner/.test(x.name) ||\n"
- "/_lodeRunner/.test(x.name) ||\n"
+ "if ( ( /[\\/\\\\]_/.test(x.name) ) ||\n"
+ "( ! /\\.js$/.test(x.name ) ) ||\n"
  "( x.name in skipTests ) ||\n"
  "( x.name in serialTests ) ||\n"
  "! /\\.js$/.test(x.name ) ){\n"
@@ -356,6 +409,8 @@ const char * jsconcatcode =
  "var nFailed = 0;\n"
  "runners.forEach( function( x ) { if( !x.returnData() ) { ++nFailed; } } );\n"
  "assert.eq( 0, nFailed, msg );}}\n"
+ "tojsononeline = function( x ){\n"
+ "return tojson( x , \" \" , true );}\n"
  "tojson = function( x, indent , nolint ){\n"
  "if ( x === null )\n"
  "return \"null\";\n"
@@ -363,14 +418,24 @@ const char * jsconcatcode =
  "return \"undefined\";\n"
  "if (!indent)\n"
  "indent = \"\";\n"
- "switch ( typeof x ){\n"
+ "switch ( typeof x ) {\n"
  "case \"string\": {\n"
  "var s = \"\\\"\";\n"
  "for ( var i=0; i<x.length; i++ ){\n"
- "if ( x[i] == '\"' ){\n"
- "s += \"\\\\\\\"\";}\n"
- "else\n"
- "s += x[i];}\n"
+ "switch (x[i]){\n"
+ "case '\"': s += '\\\\\"'; break;\n"
+ "case '\\\\': s += '\\\\\\\\'; break;\n"
+ "case '\\b': s += '\\\\b'; break;\n"
+ "case '\\f': s += '\\\\f'; break;\n"
+ "case '\\n': s += '\\\\n'; break;\n"
+ "case '\\r': s += '\\\\r'; break;\n"
+ "case '\\t': s += '\\\\t'; break;\n"
+ "default: {\n"
+ "var code = x.charCodeAt(i);\n"
+ "if (code < 0x20){\n"
+ "s += (code < 0x10 ? '\\\\u000' : '\\\\u00') + code.toString(16);\n"
+ "} else {\n"
+ "s += x[i];}}}}\n"
  "return s + \"\\\"\";}\n"
  "case \"number\":\n"
  "case \"boolean\":\n"
@@ -392,7 +457,7 @@ const char * jsconcatcode =
  "indent = \"\";\n"
  "if ( typeof( x.tojson ) == \"function\" && x.tojson != tojson ) {\n"
  "return x.tojson(indent,nolint);}\n"
- "if ( typeof( x.constructor.tojson ) == \"function\" && x.constructor.tojson != tojson ) {\n"
+ "if ( x.constructor && typeof( x.constructor.tojson ) == \"function\" && x.constructor.tojson != tojson ) {\n"
  "return x.constructor.tojson( x, indent , nolint );}\n"
  "if ( x.toString() == \"[object MaxKey]\" )\n"
  "return \"{ $maxKey : 1 }\";\n"
@@ -433,6 +498,8 @@ const char * jsconcatcode =
  "db.resetError();}}\n"
  "printjson = function(x){\n"
  "print( tojson( x ) );}\n"
+ "printjsononeline = function(x){\n"
+ "print( tojsononeline( x ) );}\n"
  "shellPrintHelper = function( x ){\n"
  "if ( typeof( x ) == \"undefined\" ){\n"
  "if ( typeof( db ) != \"undefined\" && db.getLastError ){\n"
@@ -440,6 +507,8 @@ const char * jsconcatcode =
  "if ( e != null )\n"
  "print( e );}\n"
  "return;}\n"
+ "if ( x == __magicNoPrint )\n"
+ "return;\n"
  "if ( x == null ){\n"
  "print( \"null\" );\n"
  "return;}\n"
@@ -453,6 +522,11 @@ const char * jsconcatcode =
  "print( x.tojson() );\n"
  "else\n"
  "print( tojson( x ) );}\n"
+ "shellAutocomplete = function( prefix ){\n"
+ "var a = [];\n"
+ "//a.push( prefix + \"z\" )\n"
+ "//a.push( prefix + \"y\" )\n"
+ "__autocomplete__ = a;}\n"
  "shellHelper = function( command , rest , shouldPrint ){\n"
  "command = command.trim();\n"
  "var args = rest.trim().replace(/;$/,\"\").split( \"\\s+\" );\n"
@@ -462,18 +536,6 @@ const char * jsconcatcode =
  "if ( shouldPrint ){\n"
  "shellPrintHelper( res );}\n"
  "return res;}\n"
- "help = shellHelper.help = function(){\n"
- "print( \"HELP\" );\n"
- "print( \"\\t\" + \"show dbs                     show database names\");\n"
- "print( \"\\t\" + \"show collections             show collections in current database\");\n"
- "print( \"\\t\" + \"show users                   show users in current database\");\n"
- "print( \"\\t\" + \"show profile                 show most recent system.profile entries with time >= 1ms\");\n"
- "print( \"\\t\" + \"use <db name>                set curent database to <db name>\" );\n"
- "print( \"\\t\" + \"db.help()                    help on DB methods\");\n"
- "print( \"\\t\" + \"db.foo.help()                help on collection methods\");\n"
- "print( \"\\t\" + \"db.foo.find()                list objects in collection foo\" );\n"
- "print( \"\\t\" + \"db.foo.find( { a : 1 } )     list objects in foo where a == 1\" );\n"
- "print( \"\\t\" + \"it                           result of the last line evaluated; use to further iterate\");}\n"
  "shellHelper.use = function( dbname ){\n"
  "db = db.getMongo().getDB( dbname );\n"
  "print( \"switched to db \" + db.getName() );}\n"
@@ -546,7 +608,8 @@ const char * jsconcatcode =
  "this._data[k].forEach( function(z){ all.push( z.value ); } );}\n"
  "return all;}\n"
  "if ( typeof( gc ) == \"undefined\" ){\n"
- "gc = function(){}}\n"
+ "gc = function(){\n"
+ "print( \"warning: using noop gc()\" );}}\n"
  "Math.sigFig = function( x , N ){\n"
  "if ( ! N ){\n"
  "N = 3;}\n"
@@ -568,6 +631,117 @@ const char * jsconcatcode =
  "for ( var i in inprog ) {\n"
  "if ( uris[ u ] == inprog[ i ].client ) {\n"
  "db.killOp( inprog[ i ].opid );}}}}\n"
+ "Geo = {};\n"
+ "Geo.distance = function( a , b ){\n"
+ "var ax = null;\n"
+ "var ay = null;\n"
+ "var bx = null;\n"
+ "var by = null;\n"
+ "for ( var key in a ){\n"
+ "if ( ax == null )\n"
+ "ax = a[key];\n"
+ "else if ( ay == null )\n"
+ "ay = a[key];}\n"
+ "for ( var key in b ){\n"
+ "if ( bx == null )\n"
+ "bx = b[key];\n"
+ "else if ( by == null )\n"
+ "by = b[key];}\n"
+ "return Math.sqrt( Math.pow( by - ay , 2 ) +\n"
+ "Math.pow( bx - ax , 2 ) );}\n"
+ "rs = function () { return \"try rs.help()\"; }\n"
+ "rs.help = function () {\n"
+ "print(\"\\trs.status()                     { replSetGetStatus : 1 } checks repl set status\");\n"
+ "print(\"\\trs.initiate()                   { replSetInitiate : null } initiates set with default settings\");\n"
+ "print(\"\\trs.initiate(cfg)                { replSetInitiate : cfg } initiates set with configuration cfg\");\n"
+ "print(\"\\trs.add(hostportstr)             add a new member to the set with default attributes\");\n"
+ "print(\"\\trs.add(membercfgobj)            add a new member to the set with extra attributes\");\n"
+ "print(\"\\trs.addArb(hostportstr)          add a new member which is arbiterOnly:true\");\n"
+ "print(\"\\trs.stepDown()                   step down as primary (momentarily)\");\n"
+ "print(\"\\trs.conf()                       return configuration from local.system.replset\");\n"
+ "print(\"\\trs.slaveOk()                    shorthand for db.getMongo().setSlaveOk()\");\n"
+ "print();\n"
+ "print(\"\\tdb.isMaster()                   check who is primary\");\n"
+ "print();\n"
+ "print(\"\\tsee also http://<mongod_host>:28017/_replSet for additional diagnostic info\");}\n"
+ "rs.slaveOk = function () { return db.getMongo().setSlaveOk(); }\n"
+ "rs.status = function () { return db._adminCommand(\"replSetGetStatus\"); }\n"
+ "rs.isMaster = function () { return db.isMaster(); }\n"
+ "rs.initiate = function (c) { return db._adminCommand({ replSetInitiate: c }); }\n"
+ "rs.add = function (hostport, arb) {\n"
+ "var cfg = hostport;\n"
+ "var local = db.getSisterDB(\"local\");\n"
+ "assert(local.system.replset.count() <= 1, \"error: local.system.replset has unexpected contents\");\n"
+ "var c = local.system.replset.findOne();\n"
+ "assert(c, \"no config object retrievable from local.system.replset\");\n"
+ "c.version++;\n"
+ "var max = 0;\n"
+ "for (var i in c.members)\n"
+ "if (c.members[i]._id > max) max = c.members[i]._id;\n"
+ "if (isString(hostport)) {\n"
+ "cfg = { _id: max + 1, host: hostport };\n"
+ "if (arb)\n"
+ "cfg.arbiterOnly = true;}\n"
+ "c.members.push(cfg);\n"
+ "return db._adminCommand({ replSetReconfig: c });}\n"
+ "rs.stepDown = function () { return db._adminCommand({ replSetStepDown:true}); }\n"
+ "rs.addArb = function (hn) { return this.add(hn, true); }\n"
+ "rs.conf = function () { return db.getSisterDB(\"local\").system.replset.findOne(); }\n"
+ "help = shellHelper.help = function (x) {\n"
+ "if (x == \"connect\") {\n"
+ "print(\"\\nNormally one specifies the server on the mongo shell command line.  Run mongo --help to see those options.\");\n"
+ "print(\"Additional connections may be opened:\\n\");\n"
+ "print(\"    var x = new Mongo('host[:port]');\");\n"
+ "print(\"    var mydb = x.getDB('mydb');\");\n"
+ "print(\"  or\");\n"
+ "print(\"    var mydb = connect('host[:port]/mydb');\");\n"
+ "print(\"\\nNote: the REPL prompt only auto-reports getLastError() for the shell command line connection.\\n\");\n"
+ "return;}\n"
+ "if (x == \"misc\") {\n"
+ "print(\"\\tb = new BinData(subtype,base64str)  create a BSON BinData value\");\n"
+ "print(\"\\tb.subtype()                         the BinData subtype (0..255)\");\n"
+ "print(\"\\tb.length()                          length of the BinData data in bytes\");\n"
+ "print(\"\\tb.hex()                             the data as a hex encoded string\");\n"
+ "print(\"\\tb.base64()                          the data as a base 64 encoded string\");\n"
+ "print(\"\\tb.toString()\");\n"
+ "return;}\n"
+ "if (x == \"admin\") {\n"
+ "print(\"\\tls([path])                      list files\");\n"
+ "print(\"\\tpwd()                           returns current directory\");\n"
+ "print(\"\\tlistFiles([path])               returns file list\");\n"
+ "print(\"\\thostname()                      returns name of this host\");\n"
+ "print(\"\\tcat(fname)                      returns contents of text file as a string\");\n"
+ "print(\"\\tremoveFile(f)                   delete a file\");\n"
+ "print(\"\\tload(jsfilename)                load and execute a .js file\");\n"
+ "print(\"\\trun(program[, args...])         spawn a program and wait for its completion\");\n"
+ "print(\"\\tsleep(m)                        sleep m milliseconds\");\n"
+ "print(\"\\tgetMemInfo()                    diagnostic\");\n"
+ "return;}\n"
+ "if (x == \"test\") {\n"
+ "print(\"\\tstartMongodEmpty(args)        DELETES DATA DIR and then starts mongod\");\n"
+ "print(\"\\t                              returns a connection to the new server\");\n"
+ "print(\"\\tstartMongodTest()             DELETES DATA DIR\");\n"
+ "print(\"\\t                              automatically picks port #s starting at 27000 and increasing\");\n"
+ "print(\"\\t                              or you can specify the port as the first arg\");\n"
+ "print(\"\\t                              dir is /data/db/<port>/ if not specified as the 2nd arg\");\n"
+ "print(\"\\t                              returns a connection to the new server\");\n"
+ "return;}\n"
+ "print(\"\\t\" + \"db.help()                    help on db methods\");\n"
+ "print(\"\\t\" + \"db.mycoll.help()             help on collection methods\");\n"
+ "print(\"\\t\" + \"rs.help()                    help on replica set methods\");\n"
+ "print(\"\\t\" + \"help connect                 connecting to a db help\");\n"
+ "print(\"\\t\" + \"help admin                   administrative help\");\n"
+ "print(\"\\t\" + \"help misc                    misc things to know\");\n"
+ "print();\n"
+ "print(\"\\t\" + \"show dbs                     show database names\");\n"
+ "print(\"\\t\" + \"show collections             show collections in current database\");\n"
+ "print(\"\\t\" + \"show users                   show users in current database\");\n"
+ "print(\"\\t\" + \"show profile                 show most recent system.profile entries with time >= 1ms\");\n"
+ "print(\"\\t\" + \"use <db_name>                set current database\");\n"
+ "print(\"\\t\" + \"db.foo.find()                list objects in collection foo\");\n"
+ "print(\"\\t\" + \"db.foo.find( { a : 1 } )     list objects in foo where a == 1\");\n"
+ "print(\"\\t\" + \"it                           result of the last line evaluated; use to further iterate\");\n"
+ "print(\"\\t\" + \"exit                         quit the mongo shell\");}\n"
  "if ( typeof DB == \"undefined\" ){\n"
  "DB = function( mongo , name ){\n"
  "this._mongo = mongo;\n"
@@ -679,7 +853,7 @@ const char * jsconcatcode =
  "print(\"\\tdb.commandHelp(name) returns the help for the command\");\n"
  "print(\"\\tdb.copyDatabase(fromdb, todb, fromhost)\");\n"
  "print(\"\\tdb.createCollection(name, { size : ..., capped : ..., max : ... } )\");\n"
- "print(\"\\tdb.currentOp() displays the current operation in the db\" );\n"
+ "print(\"\\tdb.currentOp() displays the current operation in the db\");\n"
  "print(\"\\tdb.dropDatabase()\");\n"
  "print(\"\\tdb.eval(func, args) run code server-side\");\n"
  "print(\"\\tdb.getCollection(cname) same as db['cname'] or db.cname\");\n"
@@ -692,9 +866,11 @@ const char * jsconcatcode =
  "print(\"\\tdb.getPrevError()\");\n"
  "print(\"\\tdb.getProfilingLevel()\");\n"
  "print(\"\\tdb.getReplicationInfo()\");\n"
- "print(\"\\tdb.getSisterDB(name) get the db at the same server as this onew\");\n"
- "print(\"\\tdb.killOp(opid) kills the current operation in the db\" );\n"
- "print(\"\\tdb.printCollectionStats()\" );\n"
+ "print(\"\\tdb.getSisterDB(name) get the db at the same server as this one\");\n"
+ "print(\"\\tdb.isMaster() check replica primary status\");\n"
+ "print(\"\\tdb.killOp(opid) kills the current operation in the db\");\n"
+ "print(\"\\tdb.listCommands() lists all the db commands\");\n"
+ "print(\"\\tdb.printCollectionStats()\");\n"
  "print(\"\\tdb.printReplicationInfo()\");\n"
  "print(\"\\tdb.printSlaveReplicationInfo()\");\n"
  "print(\"\\tdb.printShardingStatus()\");\n"
@@ -702,10 +878,13 @@ const char * jsconcatcode =
  "print(\"\\tdb.repairDatabase()\");\n"
  "print(\"\\tdb.resetError()\");\n"
  "print(\"\\tdb.runCommand(cmdObj) run a database command.  if cmdObj is a string, turns it into { cmdObj : 1 }\");\n"
+ "print(\"\\tdb.serverStatus()\");\n"
  "print(\"\\tdb.setProfilingLevel(level,<slowms>) 0=off 1=slow 2=all\");\n"
  "print(\"\\tdb.shutdownServer()\");\n"
  "print(\"\\tdb.stats()\");\n"
- "print(\"\\tdb.version() current version of the server\" );}\n"
+ "print(\"\\tdb.version() current version of the server\");\n"
+ "print(\"\\tdb.getMongo().setSlaveOk() allow queries on a replication slave server\");\n"
+ "return __magicNoPrint;}\n"
  "DB.prototype.printCollectionStats = function(){\n"
  "var mydb = this;\n"
  "this.getCollectionNames().forEach(\n"
@@ -776,13 +955,18 @@ const char * jsconcatcode =
  "return this.runCommand( { reseterror : 1 } );}\n"
  "DB.prototype.forceError = function(){\n"
  "return this.runCommand( { forceerror : 1 } );}\n"
- "DB.prototype.getLastError = function(){\n"
- "var res = this.runCommand( { getlasterror : 1 } );\n"
+ "DB.prototype.getLastError = function( w , wtimeout ){\n"
+ "var res = this.getLastErrorObj( w , wtimeout );\n"
  "if ( ! res.ok )\n"
  "throw \"getlasterror failed: \" + tojson( res );\n"
  "return res.err;}\n"
- "DB.prototype.getLastErrorObj = function(){\n"
- "var res = this.runCommand( { getlasterror : 1 } );\n"
+ "DB.prototype.getLastErrorObj = function( w , wtimeout ){\n"
+ "var cmd = { getlasterror : 1 };\n"
+ "if ( w ){\n"
+ "cmd.w = w;\n"
+ "if ( wtimeout )\n"
+ "cmd.wtimeout = wtimeout;}\n"
+ "var res = this.runCommand( cmd );\n"
  "if ( ! res.ok )\n"
  "throw \"getlasterror failed: \" + tojson( res );\n"
  "return res;}\n"
@@ -797,18 +981,18 @@ const char * jsconcatcode =
  "DB.prototype.getCollectionNames = function(){\n"
  "var all = [];\n"
  "var nsLength = this._name.length + 1;\n"
- "this.getCollection( \"system.namespaces\" ).find().sort({name:1}).forEach(\n"
- "function(z){\n"
- "var name = z.name;\n"
- "if ( name.indexOf( \"$\" ) >= 0 && name != \"local.oplog.$main\" )\n"
- "return;\n"
+ "var c = this.getCollection( \"system.namespaces\" ).find();\n"
+ "while ( c.hasNext() ){\n"
+ "var name = c.next().name;\n"
+ "if ( name.indexOf( \"$\" ) >= 0 && name.indexOf( \".oplog.$\" ) < 0 )\n"
+ "continue;\n"
  "all.push( name.substring( nsLength ) );}\n"
- ");\n"
- "return all;}\n"
+ "return all.sort();}\n"
  "DB.prototype.tojson = function(){\n"
  "return this._name;}\n"
  "DB.prototype.toString = function(){\n"
  "return this._name;}\n"
+ "DB.prototype.isMaster = function () { return this.runCommand(\"isMaster\"); }\n"
  "DB.prototype.currentOp = function(){\n"
  "return db.$cmd.sys.inprog.findOne();}\n"
  "DB.prototype.currentOP = DB.prototype.currentOp;\n"
@@ -865,13 +1049,17 @@ const char * jsconcatcode =
  "print(\"now:                     \" + result.now);}\n"
  "DB.prototype.printSlaveReplicationInfo = function() {\n"
  "function g(x) {\n"
+ "assert( x , \"how could this be null (printSlaveReplicationInfo gx)\" )\n"
  "print(\"source:   \" + x.host);\n"
+ "if ( x.syncedTo ){\n"
  "var st = new Date( DB.tsToSeconds( x.syncedTo ) * 1000 );\n"
  "var now = new Date();\n"
- "print(\"syncedTo: \" + st.toString() );\n"
+ "print(\"\\t syncedTo: \" + st.toString() );\n"
  "var ago = (now-st)/1000;\n"
  "var hrs = Math.round(ago/36)/100;\n"
- "print(\"          = \" + Math.round(ago) + \"secs ago (\" + hrs + \"hrs)\");}\n"
+ "print(\"\\t\\t = \" + Math.round(ago) + \"secs ago (\" + hrs + \"hrs)\");}\n"
+ "else {\n"
+ "print( \"\\t doing initial sync\" );}}\n"
  "var L = this.getSisterDB(\"local\");\n"
  "if( L.sources.count() == 0 ) {\n"
  "print(\"local.sources is empty; is this db a --slave?\");\n"
@@ -881,8 +1069,26 @@ const char * jsconcatcode =
  "return this._adminCommand( \"buildinfo\" );}\n"
  "DB.prototype.serverStatus = function(){\n"
  "return this._adminCommand( \"serverStatus\" );}\n"
+ "DB.prototype.serverCmdLineOpts = function(){\n"
+ "return this._adminCommand( \"getCmdLineOpts\" );}\n"
  "DB.prototype.version = function(){\n"
  "return this.serverBuildInfo().version;}\n"
+ "DB.prototype.listCommands = function(){\n"
+ "var x = this.runCommand( \"listCommands\" );\n"
+ "for ( var name in x.commands ){\n"
+ "var c = x.commands[name];\n"
+ "var s = name + \": \";\n"
+ "switch ( c.lockType ){\n"
+ "case -1: s += \"read-lock\"; break;\n"
+ "case  0: s += \"no-lock\"; break;\n"
+ "case  1: s += \"write-lock\"; break;\n"
+ "default: s += c.lockType;}\n"
+ "if (c.adminOnly) s += \" adminOnly \";\n"
+ "if (c.adminOnly) s += \" slaveOk \";\n"
+ "s += \"\\n  \";\n"
+ "s += c.help.replace(/\\n/g, '\\n  ');\n"
+ "s += \"\\n\";\n"
+ "print( s );}}\n"
  "DB.prototype.printShardingStatus = function(){\n"
  "printShardingStatus( this.getSisterDB( \"config\" ) );}\n"
  "if ( typeof Mongo == \"undefined\" ){\n"
@@ -921,12 +1127,13 @@ const char * jsconcatcode =
  "var c = ns.substring( idx + 1 );\n"
  "return this.getDB( db ).getCollection( c );}\n"
  "Mongo.prototype.toString = function(){\n"
- "return \"mongo connection to \" + this.host;}\n"
+ "return \"connection to \" + this.host;}\n"
+ "Mongo.prototype.tojson = Mongo.prototype.toString;\n"
  "connect = function( url , user , pass ){\n"
  "chatty( \"connecting to: \" + url )\n"
  "if ( user && ! pass )\n"
  "throw \"you specified a user and not a password.  either you need a password, or you're using the old connect api\";\n"
- "var idx = url.indexOf( \"/\" );\n"
+ "var idx = url.lastIndexOf( \"/\" );\n"
  "var db;\n"
  "if ( idx < 0 )\n"
  "db = new Mongo().getDB( url );\n"
@@ -1014,18 +1221,25 @@ const char * jsconcatcode =
  "this._batchSize = batchSize || 0;\n"
  "this._cursor = null;\n"
  "this._numReturned = 0;\n"
- "this._special = false;}\n"
+ "this._special = false;\n"
+ "this._prettyShell = false;}\n"
  "print( \"DBQuery probably won't have array access \" );}\n"
- "DBQuery.prototype.help = function(){\n"
- "print( \"DBQuery help\" );\n"
- "print( \"\\t.sort( {...} )\" )\n"
- "print( \"\\t.limit( n )\" )\n"
- "print( \"\\t.skip( n )\" )\n"
- "print( \"\\t.count() - total # of objects matching query, ignores skip,limit\" )\n"
- "print( \"\\t.size() - total # of objects cursor would return skip,limit effect this\" )\n"
- "print( \"\\t.explain()\" )\n"
- "print( \"\\t.forEach( func )\" )\n"
- "print( \"\\t.map( func )\" )}\n"
+ "DBQuery.prototype.help = function () {\n"
+ "print(\"find() modifiers\")\n"
+ "print(\"\\t.sort( {...} )\")\n"
+ "print(\"\\t.limit( n )\")\n"
+ "print(\"\\t.skip( n )\")\n"
+ "print(\"\\t.count() - total # of objects matching query, ignores skip,limit\")\n"
+ "print(\"\\t.size() - total # of objects cursor would return, honors skip,limit\")\n"
+ "print(\"\\t.explain([verbose])\")\n"
+ "print(\"\\t.hint(...)\")\n"
+ "print(\"\\t.showDiskLoc() - adds a $diskLoc field to each returned object\")\n"
+ "print(\"\\nCursor methods\");\n"
+ "print(\"\\t.forEach( func )\")\n"
+ "print(\"\\t.print() - output to console in full pretty format\")\n"
+ "print(\"\\t.map( func )\")\n"
+ "print(\"\\t.hasNext()\")\n"
+ "print(\"\\t.next()\")}\n"
  "DBQuery.prototype.clone = function(){\n"
  "var q =  new DBQuery( this._mongo , this._db , this._collection , this._ns ,\n"
  "this._query , this._fields ,\n"
@@ -1077,6 +1291,12 @@ const char * jsconcatcode =
  "throw \"error: \" + tojson( ret );\n"
  "this._numReturned++;\n"
  "return ret;}\n"
+ "DBQuery.prototype.objsLeftInBatch = function(){\n"
+ "this._exec();\n"
+ "var ret = this._cursor.objsLeftInBatch();\n"
+ "if ( ret.$err )\n"
+ "throw \"error: \" + tojson( ret );\n"
+ "return ret;}\n"
  "DBQuery.prototype.toArray = function(){\n"
  "if ( this._arr )\n"
  "return this._arr;\n"
@@ -1131,6 +1351,8 @@ const char * jsconcatcode =
  "return this._addSpecial( \"$min\" , min );}\n"
  "DBQuery.prototype.max = function( max ) {\n"
  "return this._addSpecial( \"$max\" , max );}\n"
+ "DBQuery.prototype.showDiskLoc = function() {\n"
+ "return this._addSpecial( \"$showDiskLoc\" , true);}\n"
  "DBQuery.prototype.forEach = function( func ){\n"
  "while ( this.hasNext() )\n"
  "func( this.next() );}\n"
@@ -1141,21 +1363,29 @@ const char * jsconcatcode =
  "return a;}\n"
  "DBQuery.prototype.arrayAccess = function( idx ){\n"
  "return this.toArray()[idx];}\n"
- "DBQuery.prototype.explain = function(){\n"
+ "DBQuery.prototype.explain = function (verbose) {\n"
+ "/* verbose=true --> include allPlans, oldPlan fields */\n"
  "var n = this.clone();\n"
  "n._ensureSpecial();\n"
  "n._query.$explain = true;\n"
  "n._limit = n._limit * -1;\n"
- "return n.next();}\n"
+ "var e = n.next();\n"
+ "if (!verbose) {\n"
+ "delete e.allPlans;\n"
+ "delete e.oldPlan;}\n"
+ "return e;}\n"
  "DBQuery.prototype.snapshot = function(){\n"
  "this._ensureSpecial();\n"
  "this._query.$snapshot = true;\n"
  "return this;}\n"
+ "DBQuery.prototype.pretty = function(){\n"
+ "this._prettyShell = true;\n"
+ "return this;}\n"
  "DBQuery.prototype.shellPrint = function(){\n"
  "try {\n"
  "var n = 0;\n"
- "while ( this.hasNext() && n < 20 ){\n"
- "var s = tojson( this.next() , \"\" , true );\n"
+ "while ( this.hasNext() && n < DBQuery.shellBatchSize ){\n"
+ "var s = this._prettyShell ? tojson( this.next() ) : tojson( this.next() , \"\" , true );\n"
  "print( s );\n"
  "n++;}\n"
  "if ( this.hasNext() ){\n"
@@ -1167,6 +1397,8 @@ const char * jsconcatcode =
  "print( e );}}\n"
  "DBQuery.prototype.toString = function(){\n"
  "return \"DBQuery: \" + this._ns + \" -> \" + tojson( this.query );}\n"
+ "DBQuery.shellBatchSize = 20;\n"
+ "// or db[\"colName\"]\n"
  "if ( ( typeof  DBCollection ) == \"undefined\" ){\n"
  "DBCollection = function( mongo , db , shortName , fullName ){\n"
  "this._mongo = mongo;\n"
@@ -1182,39 +1414,42 @@ const char * jsconcatcode =
  "assert( this._mongo , \"no mongo in DBCollection\" );}\n"
  "DBCollection.prototype.getName = function(){\n"
  "return this._shortName;}\n"
- "DBCollection.prototype.help = function() {\n"
+ "DBCollection.prototype.help = function () {\n"
+ "var shortName = this.getName();\n"
  "print(\"DBCollection help\");\n"
- "print(\"\\tdb.foo.count()\");\n"
- "print(\"\\tdb.foo.dataSize()\");\n"
- "print(\"\\tdb.foo.distinct( key ) - eg. db.foo.distinct( 'x' )\");\n"
- "print(\"\\tdb.foo.drop() drop the collection\");\n"
- "print(\"\\tdb.foo.dropIndex(name)\");\n"
- "print(\"\\tdb.foo.dropIndexes()\");\n"
- "print(\"\\tdb.foo.ensureIndex(keypattern,options) - options should be an object with these possible fields: name, unique, dropDups\");\n"
- "print(\"\\tdb.foo.reIndex()\");\n"
- "print(\"\\tdb.foo.find( [query] , [fields]) - first parameter is an optional query filter. second parameter is optional set of fields to return.\");\n"
- "print(\"\\t                                   e.g. db.foo.find( { x : 77 } , { name : 1 , x : 1 } )\");\n"
- "print(\"\\tdb.foo.find(...).count()\");\n"
- "print(\"\\tdb.foo.find(...).limit(n)\");\n"
- "print(\"\\tdb.foo.find(...).skip(n)\");\n"
- "print(\"\\tdb.foo.find(...).sort(...)\");\n"
- "print(\"\\tdb.foo.findOne([query])\");\n"
- "print(\"\\tdb.foo.findAndModify( { update : ... , remove : bool [, query: {}, sort: {}, 'new': false] } )\");\n"
- "print(\"\\tdb.foo.getDB() get DB object associated with collection\");\n"
- "print(\"\\tdb.foo.getIndexes()\");\n"
- "print(\"\\tdb.foo.group( { key : ..., initial: ..., reduce : ...[, cond: ...] } )\");\n"
- "print(\"\\tdb.foo.mapReduce( mapFunction , reduceFunction , <optional params> )\");\n"
- "print(\"\\tdb.foo.remove(query)\");\n"
- "print(\"\\tdb.foo.renameCollection( newName , <dropTarget> ) renames the collection.\");\n"
- "print(\"\\tdb.foo.runCommand( name , <options> ) runs a db command with the given name where the 1st param is the colleciton name\" );\n"
- "print(\"\\tdb.foo.save(obj)\");\n"
- "print(\"\\tdb.foo.stats()\");\n"
- "print(\"\\tdb.foo.storageSize() - includes free space allocated to this collection\");\n"
- "print(\"\\tdb.foo.totalIndexSize() - size in bytes of all the indexes\");\n"
- "print(\"\\tdb.foo.totalSize() - storage allocated for all data and indexes\");\n"
- "print(\"\\tdb.foo.update(query, object[, upsert_bool, multi_bool])\");\n"
- "print(\"\\tdb.foo.validate() - SLOW\");\n"
- "print(\"\\tdb.foo.getShardVersion() - only for use with sharding\");}\n"
+ "print(\"\\tdb.\" + shortName + \".find().help() - show DBCursor help\");\n"
+ "print(\"\\tdb.\" + shortName + \".count()\");\n"
+ "print(\"\\tdb.\" + shortName + \".dataSize()\");\n"
+ "print(\"\\tdb.\" + shortName + \".distinct( key ) - eg. db.\" + shortName + \".distinct( 'x' )\");\n"
+ "print(\"\\tdb.\" + shortName + \".drop() drop the collection\");\n"
+ "print(\"\\tdb.\" + shortName + \".dropIndex(name)\");\n"
+ "print(\"\\tdb.\" + shortName + \".dropIndexes()\");\n"
+ "print(\"\\tdb.\" + shortName + \".ensureIndex(keypattern,options) - options should be an object with these possible fields: name, unique, dropDups\");\n"
+ "print(\"\\tdb.\" + shortName + \".reIndex()\");\n"
+ "print(\"\\tdb.\" + shortName + \".find( [query] , [fields]) - first parameter is an optional query filter. second parameter is optional set of fields to return.\");\n"
+ "print(\"\\t                                   e.g. db.\" + shortName + \".find( { x : 77 } , { name : 1 , x : 1 } )\");\n"
+ "print(\"\\tdb.\" + shortName + \".find(...).count()\");\n"
+ "print(\"\\tdb.\" + shortName + \".find(...).limit(n)\");\n"
+ "print(\"\\tdb.\" + shortName + \".find(...).skip(n)\");\n"
+ "print(\"\\tdb.\" + shortName + \".find(...).sort(...)\");\n"
+ "print(\"\\tdb.\" + shortName + \".findOne([query])\");\n"
+ "print(\"\\tdb.\" + shortName + \".findAndModify( { update : ... , remove : bool [, query: {}, sort: {}, 'new': false] } )\");\n"
+ "print(\"\\tdb.\" + shortName + \".getDB() get DB object associated with collection\");\n"
+ "print(\"\\tdb.\" + shortName + \".getIndexes()\");\n"
+ "print(\"\\tdb.\" + shortName + \".group( { key : ..., initial: ..., reduce : ...[, cond: ...] } )\");\n"
+ "print(\"\\tdb.\" + shortName + \".mapReduce( mapFunction , reduceFunction , <optional params> )\");\n"
+ "print(\"\\tdb.\" + shortName + \".remove(query)\");\n"
+ "print(\"\\tdb.\" + shortName + \".renameCollection( newName , <dropTarget> ) renames the collection.\");\n"
+ "print(\"\\tdb.\" + shortName + \".runCommand( name , <options> ) runs a db command with the given name where the first param is the collection name\");\n"
+ "print(\"\\tdb.\" + shortName + \".save(obj)\");\n"
+ "print(\"\\tdb.\" + shortName + \".stats()\");\n"
+ "print(\"\\tdb.\" + shortName + \".storageSize() - includes free space allocated to this collection\");\n"
+ "print(\"\\tdb.\" + shortName + \".totalIndexSize() - size in bytes of all the indexes\");\n"
+ "print(\"\\tdb.\" + shortName + \".totalSize() - storage allocated for all data and indexes\");\n"
+ "print(\"\\tdb.\" + shortName + \".update(query, object[, upsert_bool, multi_bool])\");\n"
+ "print(\"\\tdb.\" + shortName + \".validate() - SLOW\");\n"
+ "print(\"\\tdb.\" + shortName + \".getShardVersion() - only for use with sharding\");\n"
+ "return __magicNoPrint;}\n"
  "DBCollection.prototype.getFullName = function(){\n"
  "return this._fullName;}\n"
  "DBCollection.prototype.getDB = function(){\n"
@@ -1280,7 +1515,7 @@ const char * jsconcatcode =
  "for (var key in tmp){\n"
  "obj[key] = tmp[key];}}\n"
  "this._mongo.insert( this._fullName , obj );\n"
- "return obj._id;}\n"
+ "this._lastID = obj._id;}\n"
  "DBCollection.prototype.remove = function( t ){\n"
  "this._mongo.remove( this._fullName , this._massageObject( t ) );}\n"
  "DBCollection.prototype.update = function( query , obj , upsert , multi ){\n"
@@ -1391,7 +1626,7 @@ const char * jsconcatcode =
  "var ret = this._db.runCommand( cmd );\n"
  "if ( ! ret.ok ){\n"
  "if (ret.errmsg == \"No matching object found\"){\n"
- "return {};}\n"
+ "return null;}\n"
  "throw \"findAndModifyFailed failed: \" + tojson( ret.errmsg );}\n"
  "return ret.value;}\n"
  "DBCollection.prototype.renameCollection = function( newName , dropTarget ){\n"
@@ -1401,8 +1636,9 @@ const char * jsconcatcode =
  "DBCollection.prototype.validate = function() {\n"
  "var res = this._db.runCommand( { validate: this.getName() } );\n"
  "res.valid = false;\n"
- "if ( res.result ){\n"
- "var str = \"-\" + tojson( res.result );\n"
+ "var raw = res.result || res.raw;\n"
+ "if ( raw ){\n"
+ "var str = \"-\" + tojson( raw );\n"
  "res.valid = ! ( str.match( /exception/ ) || str.match( /corrupt/ ) );\n"
  "var p = /lastExtentSize:(\\d+)/;\n"
  "var r = p.exec( str );\n"
@@ -1470,6 +1706,7 @@ const char * jsconcatcode =
  "function( spec ){\n"
  "var coll = mydb.getCollection( shortName + \".$\" + spec.name );\n"
  "var mysize = coll.storageSize();\n"
+ "//print( coll + \"\\t\" + mysize + \"\\t\" + tojson( coll.validate() ) );\n"
  "total += coll.dataSize();}\n"
  ");\n"
  "return total;}\n"
