@@ -24,6 +24,7 @@
 #include <pcrecpp.h>
 
 #include "util/file_allocator.h"
+#include "util/password.h"
 
 using namespace std;
 using namespace mongo;
@@ -34,24 +35,28 @@ namespace mongo {
 
     CmdLine cmdLine;
 
-    Tool::Tool( string name , bool localDBAllowed , string defaultDB , string defaultCollection ) :
-        _name( name ) , _db( defaultDB ) , _coll( defaultCollection ) , _conn(0), _paired(false) {
-    
+    Tool::Tool( string name , bool localDBAllowed , string defaultDB , 
+                string defaultCollection , bool usesstdout ) :
+        _name( name ) , _db( defaultDB ) , _coll( defaultCollection ) , 
+        _usesstdout(usesstdout), _noconnection(false), _autoreconnect(false), _conn(0), _paired(false) {
+        
         _options = new po::options_description( "options" );
         _options->add_options()
             ("help","produce help message")
             ("verbose,v", "be more verbose (include multiple times for more verbosity e.g. -vvvvv)")
             ("host,h",po::value<string>(), "mongo host to connect to (\"left,right\" for pairs)" )
+            ("port",po::value<string>(), "server port. Can also use --host hostname:port" )
             ("db,d",po::value<string>(), "database to use" )
             ("collection,c",po::value<string>(), "collection to use (some commands)" )
             ("username,u",po::value<string>(), "username" )
-            ("password,p",po::value<string>(), "password" )
+            ("password,p", new PasswordValue( &_password ), "password" )
+            ("ipv6", "enable IPv6 support (disabled by default)")
             ;
         if ( localDBAllowed )
             _options->add_options()
-                ("dbpath",po::value<string>(), "directly access mongod data "
-                 "files in the given path, instead of connecting to a mongod "
-                 "instance - needs to lock the data directory, so cannot be "
+                ("dbpath",po::value<string>(), "directly access mongod database "
+                 "files in the given path, instead of connecting to a mongod  "
+                 "server - needs to lock the data directory, so cannot be "
                  "used if a mongod is currently accessing the same path" )
                 ("directoryperdb", "if dbpath specified, each db is in a separate directory" )
                 ;
@@ -71,12 +76,10 @@ namespace mongo {
             delete _conn;
     }
 
-    void Tool::printExtraHelp( ostream & out ){
-    }
-
     void Tool::printHelp(ostream &out) {
         printExtraHelp(out);
         _options->print(out);
+        printExtraHelpAfter(out);
     }
 
     int Tool::main( int argc , char ** argv ){
@@ -109,8 +112,18 @@ namespace mongo {
             return EXIT_BADOPTIONS;
         }
 
+        // hide password from ps output
+        for (int i=0; i < (argc-1); ++i){
+            if (!strcmp(argv[i], "-p") || !strcmp(argv[i], "--password")){
+                char* arg = argv[i+1];
+                while (*arg){
+                    *arg++ = 'x';
+                }
+            }
+        }
+
         if ( _params.count( "help" ) ){
-            printHelp(cerr);
+            printHelp(cout);
             return 0;
         }
 
@@ -123,37 +136,39 @@ namespace mongo {
                 logLevel = s.length();
             }
         }
+        
+        preSetup();
 
         bool useDirectClient = hasParam( "dbpath" );
-
+        
         if ( ! useDirectClient ) {
             _host = "127.0.0.1";
             if ( _params.count( "host" ) )
                 _host = _params["host"].as<string>();
 
-            if ( _host.find( "," ) == string::npos ){
-                DBClientConnection * c = new DBClientConnection();
-                _conn = c;
-
+            if ( _params.count( "port" ) )
+                _host += ':' + _params["port"].as<string>();
+            
+            if ( _noconnection ){
+                // do nothing
+            }
+            else {
                 string errmsg;
-                if ( ! c->connect( _host , errmsg ) ){
+
+                ConnectionString cs = ConnectionString::parse( _host , errmsg );
+                if ( ! cs.isValid() ){
+                    cerr << "invalid hostname [" << _host << "] " << errmsg << endl;
+                    return -1;
+                }
+                
+                _conn = cs.connect( errmsg );
+                if ( ! _conn ){
                     cerr << "couldn't connect to [" << _host << "] " << errmsg << endl;
                     return -1;
                 }
             }
-            else {
-                log(1) << "using pairing" << endl;
-                DBClientPaired * c = new DBClientPaired();
-                _paired = true;
-                _conn = c;
-
-                if ( ! c->connect( _host ) ){
-                    cerr << "couldn't connect to paired server: " << _host << endl;
-                    return -1;
-                }
-            }
-
-            cerr << "connected to: " << _host << endl;
+            
+            (_usesstdout ? cout : cerr ) << "connected to: " << _host << endl;
         }
         else {
             if ( _params.count( "directoryperdb" ) ) {
@@ -168,7 +183,7 @@ namespace mongo {
             try {
                 acquirePathLock();
             }
-            catch ( DBException& e ){
+            catch ( DBException& ){
                 cerr << endl << "If you are running a mongod on the same "
                     "path you should connect to that instead of direct data "
                     "file access" << endl << endl;
@@ -188,8 +203,13 @@ namespace mongo {
         if ( _params.count( "username" ) )
             _username = _params["username"].as<string>();
 
-        if ( _params.count( "password" ) )
-            _password = _params["password"].as<string>();
+        if ( _params.count( "password" )
+             && ( _password.empty() ) ) {
+            _password = askPassword();
+        }
+
+        if (_params.count("ipv6"))
+            enableIPv6();
 
         int ret = -1;
         try {
@@ -209,14 +229,15 @@ namespace mongo {
     }
 
     DBClientBase& Tool::conn( bool slaveIfPaired ){
-        if ( _paired && slaveIfPaired )
-            return ((DBClientPaired*)_conn)->slaveConn();
+        // TODO: _paired is deprecated
+        if ( slaveIfPaired && _conn->type() == ConnectionString::SET )
+            return ((DBClientReplicaSet*)_conn)->slaveConn();
         return *_conn;
     }
 
     void Tool::addFieldOptions(){
         add_options()
-            ("fields,f" , po::value<string>() , "comma seperated list of field names e.g. -f name,age" )
+            ("fields,f" , po::value<string>() , "comma separated list of field names e.g. -f name,age" )
             ("fieldFile" , po::value<string>() , "file with fields names - 1 per line" )
             ;
     }
@@ -230,10 +251,10 @@ namespace mongo {
             pcrecpp::StringPiece input(fields_arg);
         
             string f;
-            pcrecpp::RE re("([\\w\\.\\s]+),?" );
+            pcrecpp::RE re("([#\\w\\.\\s\\-]+),?" );
             while ( re.Consume( &input, &f ) ){
                 _fields.push_back( f );
-                b.append( f.c_str() , 1 );
+                b.append( f , 1 );
             }
         
             _fieldsObj = b.obj();
@@ -254,7 +275,7 @@ namespace mongo {
                 file.getline( line , BUF_SIZE );
                 const char * cur = line;
                 while ( isspace( cur[0] ) ) cur++;
-                if ( strlen( cur ) == 0 )
+                if ( cur[0] == '\0' )
                     continue;
 
                 _fields.push_back( cur );
@@ -286,6 +307,105 @@ namespace mongo {
         throw UserException( 9997 , (string)"auth failed: " + errmsg );
     }
 
+    BSONTool::BSONTool( const char * name , bool objcheck ) 
+        : Tool( name , true , "" , "" ) , _objcheck( objcheck ){
+        
+        add_options()
+            ("objcheck" , "validate object before inserting" )
+            ("filter" , po::value<string>() , "filter to apply before inserting" )
+            ;
+    }
+
+
+    int BSONTool::run(){
+        _objcheck = hasParam( "objcheck" );
+        
+        if ( hasParam( "filter" ) )
+            _matcher.reset( new Matcher( fromjson( getParam( "filter" ) ) ) );
+        
+        return doRun();
+    }
+
+    long long BSONTool::processFile( const path& root ){
+        string fileString = root.string();
+        
+        long long fileLength = file_size( root );
+
+        if ( fileLength == 0 ) {
+            out() << "file " << fileString << " empty, skipping" << endl;
+            return 0;
+        }
+
+
+        FILE* file = fopen( fileString.c_str() , "rb" );
+        if ( ! file ){
+            log() << "error opening file: " << fileString << endl;
+            return 0;
+        }
+
+#if !defined(__sunos__) && defined(POSIX_FADV_SEQUENTIAL)
+        posix_fadvise(fileno(file), 0, fileLength, POSIX_FADV_SEQUENTIAL);
+#endif
+
+        log(1) << "\t file size: " << fileLength << endl;
+
+        long long read = 0;
+        long long num = 0;
+        long long processed = 0;
+
+        const int BUF_SIZE = 1024 * 1024 * 5;
+        boost::scoped_array<char> buf_holder(new char[BUF_SIZE]);
+        char * buf = buf_holder.get();
+
+        ProgressMeter m( fileLength );
+
+        while ( read < fileLength ) {
+            int readlen = fread(buf, 4, 1, file);
+            int size = ((int*)buf)[0];
+            if ( size >= BUF_SIZE ){
+                cerr << "got an object of size: " << size << "  terminating..." << endl;
+            }
+            uassert( 10264 ,  "invalid object size" , size < BUF_SIZE );
+
+            readlen = fread(buf+4, size-4, 1, file);
+
+            BSONObj o( buf );
+            if ( _objcheck && ! o.valid() ){
+                cerr << "INVALID OBJECT - going try and pring out " << endl;
+                cerr << "size: " << size << endl;
+                BSONObjIterator i(o);
+                while ( i.more() ){
+                    BSONElement e = i.next();
+                    try {
+                        e.validate();
+                    }
+                    catch ( ... ){
+                        cerr << "\t\t NEXT ONE IS INVALID" << endl;
+                    }
+                    cerr << "\t name : " << e.fieldName() << " " << e.type() << endl;
+                    cerr << "\t " << e << endl;
+                }
+            }
+            
+            if ( _matcher.get() == 0 || _matcher->matches( o ) ){
+                gotObject( o );
+                processed++;
+            }
+
+            read += o.objsize();
+            num++;
+
+            m.hit( o.objsize() );
+        }
+
+        uassert( 10265 ,  "counts don't match" , m.done() == fileLength );
+        out() << "\t "  << m.hits() << " objects found" << endl;
+        if ( _matcher.get() )
+            out() << "\t "  << processed << " objects processed" << endl;
+        return processed;
+    }
+            
+
 
     void setupSignals(){}
 }
