@@ -35,7 +35,8 @@ namespace mongo {
 
     class Tool {
     public:
-        Tool( string name , bool localDBAllowed=true, string defaultDB="test" , string defaultCollection="");
+        Tool( string name , bool localDBAllowed=true, string defaultDB="test" , 
+              string defaultCollection="", bool usesstdout=true );
         virtual ~Tool();
 
         int main( int argc , char ** argv );
@@ -71,12 +72,15 @@ namespace mongo {
             }
             return _db + "." + _coll;
         }
+        
+        virtual void preSetup(){}
 
         virtual int run() = 0;
 
         virtual void printHelp(ostream &out);
 
-        virtual void printExtraHelp( ostream & out );
+        virtual void printExtraHelp( ostream & out ){}
+        virtual void printExtraHelpAfter( ostream & out ){}
 
     protected:
 
@@ -90,6 +94,10 @@ namespace mongo {
 
         string _username;
         string _password;
+        
+        bool _usesstdout;
+        bool _noconnection;
+        bool _autoreconnect;
 
         void addFieldOptions();
         void needFields();
@@ -98,8 +106,10 @@ namespace mongo {
         BSONObj _fieldsObj;
 
         
-    private:
         string _host;
+
+    protected:
+
         mongo::DBClientBase * _conn;
         bool _paired;
 
@@ -111,4 +121,20 @@ namespace mongo {
 
     };
 
+    class BSONTool : public Tool {
+        bool _objcheck;
+        auto_ptr<Matcher> _matcher;
+        
+    public:
+        BSONTool( const char * name , bool objcheck = false );
+        
+        virtual int doRun() = 0;
+        virtual void gotObject( const BSONObj& obj ) = 0;
+        
+        virtual int run();
+
+        long long processFile( const path& file );
+        
+    };
+
 }
