@@ -31,7 +31,9 @@ namespace mongo {
         return x;
     }
     
-#define malloc mongo::ourmalloc
-#define realloc mongo::ourrealloc
+#define MONGO_malloc mongo::ourmalloc
+#define malloc MONGO_malloc
+#define MONGO_realloc mongo::ourrealloc
+#define realloc MONGO_realloc
     
 } // namespace mongo
