@@ -15,10 +15,20 @@
  *    limitations under the License.
  */
 
-#include "stdafx.h"
+#include "pch.h"
 #include "assert_util.h"
 #include "assert.h"
 #include "file.h"
+#include <cmath>
+using namespace std;
+
+#ifndef _WIN32
+#include <cxxabi.h>
+#include <sys/file.h>
+#endif
+
+//#include "../bson/bson.h"
+#include "../db/jsobj.h"
 
 namespace mongo {
 
@@ -41,6 +51,17 @@ namespace mongo {
         if ( newvalue >= max )
             rollover();
     }
+
+    void ExceptionInfo::append( BSONObjBuilder& b , const char * m , const char * c ) const {
+        if ( msg.empty() )
+            b.append( m , "unknown assertion" );
+        else
+            b.append( m , msg );
+        
+        if ( code )
+            b.append( c , code );
+    }
+
     
 	string getDbContext();
 	
@@ -63,8 +84,7 @@ namespace mongo {
         lastAssert[0].set(msg, getDbContext().c_str(), file, line);
         stringstream temp;
         temp << "assertion " << file << ":" << line;
-        AssertionException e;
-        e.msg = temp.str();
+        AssertionException e(temp.str(),0);
         breakpoint();
         throw e;
     }
@@ -74,13 +94,8 @@ namespace mongo {
         raiseError(0,msg);
     }
 
-    int uacount = 0;
     void uasserted(int msgid, const char *msg) {
         assertionCount.condrollover( ++assertionCount.user );
-        if ( ++uacount < 100 )
-            log() << "User Exception " << msgid << ":" << msg << endl;
-        else
-            RARELY log() << "User Exception " << msg << endl;
         lastAssert[3].set(msg, getDbContext().c_str(), "", 0);
         raiseError(msgid,msg);
         throw UserException(msgid, msg);
@@ -88,11 +103,19 @@ namespace mongo {
 
     void msgasserted(int msgid, const char *msg) {
         assertionCount.condrollover( ++assertionCount.warning );
-        log() << "Assertion: " << msgid << ":" << msg << endl;
+        tlog() << "Assertion: " << msgid << ":" << msg << endl;
         lastAssert[2].set(msg, getDbContext().c_str(), "", 0);
         raiseError(msgid,msg && *msg ? msg : "massert failure");
         breakpoint();
-        printStackTrace(); // TEMP?? should we get rid of this?  TODO
+        printStackTrace();
+        throw MsgAssertionException(msgid, msg);
+    }
+
+    void msgassertedNoTrace(int msgid, const char *msg) {
+        assertionCount.condrollover( ++assertionCount.warning );
+        log() << "Assertion: " << msgid << ":" << msg << endl;
+        lastAssert[2].set(msg, getDbContext().c_str(), "", 0);
+        raiseError(msgid,msg && *msg ? msg : "massert failure");
         throw MsgAssertionException(msgid, msg);
     }
 
@@ -100,12 +123,11 @@ namespace mongo {
         stringstream ss;
         // errno might not work on all systems for streams
         // if it doesn't for a system should deal with here
-        ss << msg << " stream invalie: " << OUTPUT_ERRNO;
+        ss << msg << " stream invalid: " << errnoWithDescription();
         throw UserException( code , ss.str() );
     }
     
-    
-    mongo::mutex *Assertion::_mutex = new mongo::mutex();
+    mongo::mutex *Assertion::_mutex = new mongo::mutex("Assertion");
 
     string Assertion::toString() {
         if( _mutex == 0 )
@@ -125,89 +147,31 @@ namespace mongo {
         return ss.str();
     }	
 
-    
-    class LoggingManager {
-    public:
-        LoggingManager()
-            : _enabled(0) , _file(0) {
-        }
-        
-        void start( const string& lp , bool append ){
-            uassert( 10268 ,  "LoggingManager already started" , ! _enabled );
-            _append = append;
-
-            // test path
-            FILE * test = fopen( lp.c_str() , _append ? "a" : "w" );
-            if ( ! test ){
-                cout << "can't open [" << lp << "] for log file" << endl;
-                dbexit( EXIT_BADOPTIONS );
-                assert( 0 );
-            }
-            fclose( test );
-            
-            _path = lp;
-            _enabled = 1;
-            rotate();
-        }
-        
-        void rotate(){
-            if ( ! _enabled ){
-                cout << "LoggingManager not enabled" << endl;
-                return;
-            }
-
-            if ( _file ){
-#ifdef _WIN32
-                cout << "log rotation doesn't work on windows" << endl;
-                return;
-#else
-                struct tm t;
-                localtime_r( &_opened , &t );
-                
-                stringstream ss;
-                ss << _path << "." << ( 1900 + t.tm_year ) << "-" << t.tm_mon << "-" << t.tm_mday 
-                   << "_" << t.tm_hour << "-" << t.tm_min << "-" << t.tm_sec;
-                string s = ss.str();
-                rename( _path.c_str() , s.c_str() );
-#endif
-            }
-        
-            _file = freopen( _path.c_str() , _append ? "a" : "w"  , stdout );
-            if ( ! _file ){
-                cerr << "can't open: " << _path.c_str() << " for log file" << endl;
-                dbexit( EXIT_BADOPTIONS );
-                assert(0);
-            }
-            _opened = time(0);
-        }
-        
-    private:
-        
-        bool _enabled;
-        string _path;
-        bool _append;
-        
-        FILE * _file;
-        time_t _opened;
-        
-    } loggingManager;
-
-    void initLogging( const string& lp , bool append ){
-        cout << "all output going to: " << lp << endl;
-        loggingManager.start( lp , append );
-    }
-
-    void rotateLogs( int signal ){
-        loggingManager.rotate();
-    }
-
-    string errnostring( const char * prefix ){
+    string errnoWithPrefix( const char * prefix ){
         stringstream ss;
         if ( prefix )
             ss << prefix << ": ";
-        ss << OUTPUT_ERRNO;
+        ss << errnoWithDescription();
         return ss.str();
     }
 
+
+    string demangleName( const type_info& typeinfo ){
+#ifdef _WIN32
+        return typeinfo.name();
+#else
+        int status;
+        
+        char * niceName = abi::__cxa_demangle(typeinfo.name(), 0, 0, &status);
+        if ( ! niceName )
+            return typeinfo.name();
+        
+        string s = niceName;
+        free(niceName);
+        return s;
+#endif
+    }
+
+
 }
 
