@@ -22,7 +22,7 @@
 
 #pragma once
 
-#include "../stdafx.h"
+#include "../pch.h"
 #include <map>
 
 namespace mongo {
@@ -36,7 +36,8 @@ namespace mongo {
 
     template <
     class Key,
-    class Type
+    class Type,
+    class PTR
     >
     class HashTable : boost::noncopyable {
     public:
@@ -51,10 +52,15 @@ namespace mongo {
             void setUnused() {
                 hash = 0;
             }
-        } *nodes;
+        };
+        PTR _buf;
         int n;
         int maxChain;
 
+        Node& nodes(int i) {
+            return *((Node*) _buf.at(i * sizeof(Node), sizeof(Node)));
+        }
+
         int _find(const Key& k, bool& found) {
             found = false;
             int h = k.hash();
@@ -63,12 +69,12 @@ namespace mongo {
             int chain = 0;
             int firstNonUsed = -1;
             while ( 1 ) {
-                if ( !nodes[i].inUse() ) {
+                if ( !nodes(i).inUse() ) {
                     if ( firstNonUsed < 0 )
                         firstNonUsed = i;
                 }
 
-                if ( nodes[i].hash == h && nodes[i].k == k ) {
+                if ( nodes(i).hash == h && nodes(i).k == k ) {
                     if ( chain >= 200 )
                         out() << "warning: hashtable " << name << " long chain " << endl;
                     found = true;
@@ -92,24 +98,28 @@ namespace mongo {
 
     public:
         /* buf must be all zeroes on initialization. */
-        HashTable(void *buf, int buflen, const char *_name) : name(_name) {
+        HashTable(PTR buf, int buflen, const char *_name) : name(_name) {
             int m = sizeof(Node);
             // out() << "hashtab init, buflen:" << buflen << " m:" << m << endl;
             n = buflen / m;
             if ( (n & 1) == 0 )
                 n--;
             maxChain = (int) (n * 0.05);
-            nodes = (Node *) buf;
+            _buf = buf;
+            //nodes = (Node *) buf;
+
+            if ( sizeof(Node) != 628 ){
+                out() << "HashTable() " << _name << " sizeof(node):" << sizeof(Node) << " n:" << n << " sizeof(Key): " << sizeof(Key) << " sizeof(Type):" << sizeof(Type) << endl;
+                assert( sizeof(Node) == 628 );
+            }
 
-            assert( sizeof(Node) == 628 );
-            //out() << "HashTable() " << _name << " sizeof(node):" << sizeof(Node) << " n:" << n << endl;
         }
 
         Type* get(const Key& k) {
             bool found;
             int i = _find(k, found);
             if ( found )
-                return &nodes[i].value;
+                return &nodes(i).value;
             return 0;
         }
 
@@ -117,8 +127,9 @@ namespace mongo {
             bool found;
             int i = _find(k, found);
             if ( i >= 0 && found ) {
-                nodes[i].k.kill();
-                nodes[i].setUnused();
+                Node& n = nodes(i);
+                n.k.kill();
+                n.setUnused();
             }
         }
 /*
@@ -136,24 +147,34 @@ namespace mongo {
             int i = _find(k, found);
             if ( i < 0 )
                 return false;
+            Node& n = nodes(i);
             if ( !found ) {
-                nodes[i].k = k;
-                nodes[i].hash = k.hash();
+                n.k = k;
+                n.hash = k.hash();
             }
             else {
-                assert( nodes[i].hash == k.hash() );
+                assert( n.hash == k.hash() );
             }
-            nodes[i].value = value;
+            n.value = value;
             return true;
         }
         
         typedef void (*IteratorCallback)( const Key& k , Type& v );
-        
         void iterAll( IteratorCallback callback ){
             for ( int i=0; i<n; i++ ){
-                if ( ! nodes[i].inUse() )
+                if ( ! nodes(i).inUse() )
                     continue;
-                callback( nodes[i].k , nodes[i].value );
+                callback( nodes(i).k , nodes(i).value );
+            }
+        }
+
+        // TODO: should probably use boost::bind for this, but didn't want to look at it
+        typedef void (*IteratorCallback2)( const Key& k , Type& v , void * extra );
+        void iterAll( IteratorCallback2 callback , void * extra ){
+            for ( int i=0; i<n; i++ ){
+                if ( ! nodes(i).inUse() )
+                    continue;
+                callback( nodes(i).k , nodes(i).value , extra );
             }
         }
     
