@@ -15,9 +15,10 @@
  *    limitations under the License.
  */
 
-#include "stdafx.h"
+#include "pch.h"
 #include "mmap.h"
 #include "file_allocator.h"
+#include "../db/concurrency.h"
 
 #include <errno.h>
 #include <sys/mman.h>
@@ -60,7 +61,7 @@ namespace mongo {
         
         fd = open(filename, O_RDWR | O_NOATIME);
         if ( fd <= 0 ) {
-            out() << "couldn't open " << filename << ' ' << OUTPUT_ERRNO << endl;
+            out() << "couldn't open " << filename << ' ' << errnoWithDescription() << endl;
             return 0;
         }
 
@@ -74,7 +75,7 @@ namespace mongo {
         
         view = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
         if ( view == MAP_FAILED ) {
-            out() << "  mmap() failed for " << filename << " len:" << length << " " << OUTPUT_ERRNO << endl;
+            out() << "  mmap() failed for " << filename << " len:" << length << " " << errnoWithDescription() << endl;
             if ( errno == ENOMEM ){
                 out() << "     mmap failed with out of memory, if you're using 32-bits, then you probably need to upgrade to 64" << endl;
             }
@@ -86,10 +87,15 @@ namespace mongo {
 #else
         if ( options & SEQUENTIAL ){
             if ( madvise( view , length , MADV_SEQUENTIAL ) ){
-                out() << " madvise failed for " << filename << " " << OUTPUT_ERRNO << endl;
+                out() << " madvise failed for " << filename << " " << errnoWithDescription() << endl;
             }
         }
 #endif
+
+        DEV if (! dbMutex.info().isLocked()){
+            _unlock();
+        }
+
         return view;
     }
     
@@ -97,9 +103,38 @@ namespace mongo {
         if ( view == 0 || fd == 0 )
             return;
         if ( msync(view, len, sync ? MS_SYNC : MS_ASYNC) )
-            problem() << "msync " << OUTPUT_ERRNO << endl;
+            problem() << "msync " << errnoWithDescription() << endl;
     }
     
+    class PosixFlushable : public MemoryMappedFile::Flushable {
+    public:
+        PosixFlushable( void * view , HANDLE fd , long len )
+            : _view( view ) , _fd( fd ) , _len(len){
+        }
+
+        void flush(){
+            if ( _view && _fd )
+                if ( msync(_view, _len, MS_SYNC ) )
+                    problem() << "msync " << errnoWithDescription() << endl;
+            
+        }
+
+        void * _view;
+        HANDLE _fd;
+        long _len;
+    };
+
+    MemoryMappedFile::Flushable * MemoryMappedFile::prepareFlush(){
+        return new PosixFlushable( view , fd , len );
+    }
+
+    void MemoryMappedFile::_lock() {
+        if (view) assert(mprotect(view, len, PROT_READ | PROT_WRITE) == 0);
+    }
+
+    void MemoryMappedFile::_unlock() {
+        if (view) assert(mprotect(view, len, PROT_READ) == 0);
+    }
 
 } // namespace mongo
 
