@@ -15,8 +15,9 @@
  *    limitations under the License.
  */
 
-#include "stdafx.h"
+#include "pch.h"
 #include "mmap.h"
+#include "text.h"
 #include <windows.h>
 
 namespace mongo {
@@ -40,12 +41,6 @@ namespace mongo {
             CloseHandle(fd);
         fd = 0;
     }
-
-    std::wstring toWideString(const char *s) {
-        std::basic_ostringstream<TCHAR> buf;
-        buf << s;
-        return buf.str();
-    }
     
     unsigned mapped = 0;
 
@@ -68,14 +63,14 @@ namespace mongo {
         }
 
         updateLength( filename, length );
-        std::wstring filenamew = toWideString(filename);
 
         DWORD createOptions = FILE_ATTRIBUTE_NORMAL;
         if ( options & SEQUENTIAL )
             createOptions |= FILE_FLAG_SEQUENTIAL_SCAN;
 
         fd = CreateFile(
-                 filenamew.c_str(), GENERIC_WRITE | GENERIC_READ, FILE_SHARE_READ,
+                 toNativeString(filename).c_str(),
+                 GENERIC_WRITE | GENERIC_READ, FILE_SHARE_READ,
                  NULL, OPEN_ALWAYS, createOptions , NULL);
         if ( fd == INVALID_HANDLE_VALUE ) {
             out() << "Create/OpenFile failed " << filename << ' ' << GetLastError() << endl;
@@ -92,7 +87,7 @@ namespace mongo {
 
         view = MapViewOfFile(maphandle, FILE_MAP_ALL_ACCESS, 0, 0, 0);
         if ( view == 0 ) {
-            out() << "MapViewOfFile failed " << filename << " " << OUTPUT_ERRNO << " ";
+            out() << "MapViewOfFile failed " << filename << " " << errnoWithDescription() << " ";
             out() << GetLastError();
             out() << endl;
         }
@@ -100,21 +95,47 @@ namespace mongo {
         return view;
     }
 
+    class WindowsFlushable : public MemoryMappedFile::Flushable {
+    public:
+        WindowsFlushable( void * view , HANDLE fd , string filename )
+            : _view(view) , _fd(fd) , _filename(filename){
+            
+        }
+        
+        void flush(){
+            if (!_view || !_fd) 
+                return;
+
+            bool success = FlushViewOfFile(_view, 0); // 0 means whole mapping
+            if (!success){
+                int err = GetLastError();
+                out() << "FlushViewOfFile failed " << err << " file: " << _filename << endl;
+            }
+            
+            success = FlushFileBuffers(_fd);
+            if (!success){
+                int err = GetLastError();
+                out() << "FlushFileBuffers failed " << err << " file: " << _filename << endl;
+            }
+        }
+        
+        void * _view;
+        HANDLE _fd;
+        string _filename;
+        
+    };
+    
     void MemoryMappedFile::flush(bool sync) {
         uassert(13056, "Async flushing not supported on windows", sync);
-
-        if (!view || !fd) return;
-
-        bool success = FlushViewOfFile(view, 0); // 0 means whole mapping
-        if (!success){
-            int err = GetLastError();
-            out() << "FlushViewOfFile failed " << err << " file: " << _filename << endl;
-        }
-
-        success = FlushFileBuffers(fd);
-        if (!success){
-            int err = GetLastError();
-            out() << "FlushFileBuffers failed " << err << " file: " << _filename << endl;
-        }
+        
+        WindowsFlushable f( view , fd , _filename );
+        f.flush();
     }
+
+    MemoryMappedFile::Flushable * MemoryMappedFile::prepareFlush(){
+        return new WindowsFlushable( view , fd , _filename );
+    }
+    void MemoryMappedFile::_lock() {}
+    void MemoryMappedFile::_unlock() {}
+
 } 
